package catalunya.de.ocells;

import java.util.ArrayList;

import android.app.ActionBar;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.CursorAdapter;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.SlidingDrawer;
import android.widget.TextView;
import catalunya.de.ocells.cerca.EstatCercaActivity;
import catalunya.de.ocells.cerca.defs;
import catalunya.de.ocells.cerca.pregunta;
import catalunya.de.ocells.cerca.preguntes;
import catalunya.de.ocells.cerca.opcions;
import catalunya.de.ocells.db.dbDadesOcellAdapter;
import catalunya.de.ocells.db.dbOcellsAdapter;


public class CercaInteligentActivity extends Activity {

	private preguntes mPreguntes;
	private GridView gvLlistatOpcions;
	private OpcionsAdapter mOpcionsAdapter;
	private PropostesAdapter mPropostesAdapter;
	private ListView lvLlistatPropostes;
	private ArrayList<String[]> llistaOpcions; 
	private Integer preguntaActual;
	public Integer horitzoSuperior;
	//private boolean propostesFetes = false;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_cerca_inteligent);
		
		if (android.os.Build.VERSION.SDK_INT >= 11)
			getActionBar().setDisplayHomeAsUpEnabled(true);
		final preguntes tPreguntes = (preguntes) getLastNonConfigurationInstance();
	    if (tPreguntes == null) {
	    	mPreguntes = new preguntes(this);
	    	mPreguntes.reiniciarPunts();
	    	mPreguntes.aplicarCaracteristiquesImplicites();
	    } else mPreguntes = tPreguntes;
	
	    gvLlistatOpcions = (GridView)findViewById(R.id.cerca_inteligent_llista_opcions);
	    lvLlistatPropostes = (ListView)findViewById(R.id.cerca_inteligent_llista_sliding_drawer);
	    
	    //Listener per a les opcions
	    gvLlistatOpcions.setOnItemClickListener(new OnItemClickListener() {
	          @Override
	          public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
	              String codiOpcio = (String) view.getTag();
 	              codiOpcio = mPreguntes.actualitzaPreguntes(preguntaActual, codiOpcio);
	              if (!codiOpcio.equals(defs.SENSE_RESPOSTA))
	            	  mPreguntes.actualitzaProbabilitats(preguntaActual, codiOpcio);
	        	  carregaPreguntaActual();
	          }
	    });
	    //Listener per a les propostes
	    lvLlistatPropostes.setOnItemClickListener(new OnItemClickListener() {
	          @Override
	          public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
	           	  Intent fitxaIntent = new Intent(getApplicationContext(), FitxaOcellActivity.class);
	              fitxaIntent.putExtra("id", id);
	              startActivity(fitxaIntent);
	          }
	    });
	    
	    carregaPreguntaActual();
	}
	
	@Override
	public Object onRetainNonConfigurationInstance() {
	    //Desem l'objecte per a no tronar-lo a carregar quan es destrueix
	    return mPreguntes;
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.activity_cerca_inteligent, menu);
		return true;
	}
	  
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
	      switch (item.getItemId()) {
			    case android.R.id.home:
		      		finish();
		      		return true;
	      		case R.id.cerca_inteligent_menu_estat_cerca:
		        	Intent intent = new Intent(getApplicationContext(), EstatCercaActivity.class);
		    		startActivity(intent);
		    		return true;
	      		case R.id.cerca_inteligent_menu_preferencies:
	            	Intent intent2 = new Intent(getApplicationContext(), SettingsActivity.class);
	                startActivity(intent2);            	
	            	return true;
	      		case R.id.cerca_inteligent_menu_ajuda:
	 	        	Intent i = new Intent(Intent.ACTION_VIEW);
	 	           	i.setData(Uri.parse(this.getResources().getString((R.string.url_ajuda))));
	 	           	startActivity(i);           	
	 	           	return true;
		        default:
	              return false;
	      }
	}	
	

	public void carregaPreguntaActual(){
		preguntaActual = mPreguntes.cercarPreguntaOptima();
		if (preguntaActual!=defs.SENSE_PREGUNTES)
			llistaOpcions = mPreguntes.getOpcionsPregunta(preguntaActual);
		else
			llistaOpcions = new ArrayList<String[]>();
		carregarGUI();
		carregaPropostes();
	}

	public void carregarGUI(){
		Integer nombrePreguntes = mPreguntes.preguntesFetes + 1;
		if (android.os.Build.VERSION.SDK_INT >= 11){
			ActionBar ab = getActionBar();
			ab.setSubtitle(this.getString(R.string.cerca_inteligent_titol_contador_pregunta) + " " + nombrePreguntes) ;
		}

		TextView tvPregunta = (TextView)findViewById(R.id.cerca_inteligent_pregunta);
		ImageView imatge = (ImageView)findViewById(R.id.cerca_inteligent_pregunta_imatge);
		
		tvPregunta.setText(mPreguntes.p.get(preguntaActual).text);
		String nomImatge = mPreguntes.p.get(preguntaActual).imatge;
		
		if (nomImatge != null){
			imatge.setVisibility(View.VISIBLE);
			int idImatge = getResources().getIdentifier(nomImatge, "drawable", getPackageName());
	     	imatge.setImageDrawable(this.getResources().getDrawable(idImatge));
	    }
		else
			imatge.setVisibility(View.GONE);

		mOpcionsAdapter = new OpcionsAdapter(this);
		gvLlistatOpcions.setAdapter(mOpcionsAdapter);
	
		Integer ampladaDispositiu = eines.getAmpladaDispositiu(getWindowManager());
//		if (mPreguntes.p.get(preguntaActual).nom.equals("tamanyCodi")){
//			gvLlistatOpcions.setNumColumns(1);
//			//gvLlistatOpcions.setStretchMode( GridView.NO_STRETCH ) ;
//		}
//		else{
			//gvLlistatOpcions.setNumColumns(4);
			Integer columnes = ampladaDispositiu / 150;
			gvLlistatOpcions.setNumColumns(columnes);
			
			//gvLlistatOpcions.setColumnWidth(ampladaDispositiu);
			//Toast.makeText(this, ampladaDispositiu.toString() , Toast.LENGTH_SHORT).show();
//		else
//			gvLlistatOpcions.setColumnWidth(110);
//		//	gvLlistatOpcions.setNumColumns(3);
		
//		}
	}

	private void carregaPropostes() {
		SlidingDrawer mSlidingDrawer = (SlidingDrawer)findViewById(R.id.cerca_inteligent_sliding_drawer);
		TextView tvInfo = (TextView)findViewById(R.id.cerca_inteligent_info_propostes);
		boolean mostrarPropostes = false;	
		horitzoSuperior = mPreguntes.cercarHoritzo(defs.PERCENT_HORITZO_SUPERIOR);
		Integer horitzoInferior = mPreguntes.cercarHoritzo(defs.PERCENT_HORITZO_INFERIOR);
		dbDadesOcellAdapter mdbDadesOcellAdapter = new dbDadesOcellAdapter(this);
		mdbDadesOcellAdapter.open();
		Integer especiesHoritzoSuperior = mdbDadesOcellAdapter.getNombreEspeciesHoritzo(horitzoSuperior);
		Integer especiesHoritzoInferior = mdbDadesOcellAdapter.getNombreEspeciesHoritzo(horitzoInferior);
		Integer especiesIgualsHoritzoSuperior = mdbDadesOcellAdapter.getNombreEspeciesIgualsHoritzo(horitzoSuperior);
		Integer especiesIgualsHoritzoInferior = mdbDadesOcellAdapter.getNombreEspeciesIgualsHoritzo(horitzoInferior);


		//Definim text de informaci de la proposta...
		if (especiesIgualsHoritzoInferior==1 ){
			tvInfo.setText(R.string.cerca_inteligent_info_propostes_molt_probable);
			mostrarPropostes = true;
		} else if (especiesIgualsHoritzoSuperior==1){
	    	tvInfo.setText(R.string.cerca_inteligent_info_propostes_probable);
	    	mostrarPropostes = true;
		} else if (especiesIgualsHoritzoSuperior<4){
	    	tvInfo.setText(R.string.cerca_inteligent_info_propostes_poc_probable);
		} else
	    	tvInfo.setText(R.string.cerca_inteligent_info_propostes_no_determinat);
	    
		//Si hi ha propostes, les mostrem
		if (preguntaActual==defs.SENSE_PREGUNTES || mostrarPropostes){
			if (!mSlidingDrawer.isOpened())
				mSlidingDrawer.animateOpen();
		}else{
			//Si no hi han propostes a fer, les amaguem
			if (mSlidingDrawer.isOpened())
			mSlidingDrawer.animateClose();
			
		}
	
		Cursor cursor = mdbDadesOcellAdapter.getEstatCerca(especiesHoritzoInferior);
		mPropostesAdapter = new PropostesAdapter(this, cursor);
	    lvLlistatPropostes.setAdapter(mPropostesAdapter);
		mdbDadesOcellAdapter.close();
	  	   
	}

	
   //Classe per a les opcions
   public class OpcionsAdapter extends BaseAdapter {
	   private Context mContext;
	   private LayoutInflater mInflater=null;

        public OpcionsAdapter(Context c) {
        	super();
        	this.mContext = (Activity) c;
        	mInflater = LayoutInflater.from(mContext);
        }

        public int getCount() {
            return llistaOpcions.size();
        }

        public Object getItem(int position) {
            return position;
        }

        public long getItemId(int position) {
            return position;
        }
        
     
        public View getView(int position, View convertView, ViewGroup parent) {
        	if (convertView == null) {
        		  //mInflater = mContext.getLayoutInflater();
	    	      convertView = mInflater.inflate(R.layout.row_opcio_cerca_inteligent, null);
	    	}
        	TextView nomOpcio = (TextView)convertView.findViewById(R.id.row_opcio_cerca_inteligent_text);
	        ImageView imatge = (ImageView)convertView.findViewById(R.id.row_opcio_cerca_inteligent_icona);
        	
	        nomOpcio.setText(llistaOpcions.get(position)[opcions.TEXT]);
	        String recurs = llistaOpcions.get(position)[opcions.RECURS];
            Integer idImatge = mContext.getResources().getIdentifier(recurs, "drawable", mContext.getPackageName());
            
            if (idImatge != 0){
            	imatge.setImageResource(idImatge);
            	imatge.setTag(llistaOpcions.get(position)[opcions.CODI]);
            	imatge.setAdjustViewBounds(true);
            	imatge.setScaleType(ImageView.ScaleType.FIT_CENTER);
            	imatge.setPadding(4, 4, 4, 0);
            }
	        //imatge.setImageDrawable(context.getResources().getDrawable(id_imatge));
	        convertView.setTag(llistaOpcions.get(position)[opcions.CODI]);
	        return convertView;
        	
//        	ImageView imageView;
//            TextView textView;
//            //if (convertView == null) {
//                imageView = new ImageView(mContext);
//                
//                String recurs = llistaOpcions.get(position)[opcions.RECURS];
//                Integer idImatge = mContext.getResources().getIdentifier(recurs, "drawable", mContext.getPackageName());
//                if (idImatge != 0){
//                	imageView.setImageResource(idImatge);
//                	imageView.setTag(llistaOpcions.get(position)[opcions.CODI]);
//                    imageView.setAdjustViewBounds(false);
//                    imageView.setScaleType(ImageView.ScaleType.FIT_CENTER);
//                    imageView.setPadding(4, 4, 4, 4);
//                	return imageView;
//                } else{
//                	textView = new TextView(mContext);
//                	textView.setText(recurs);
//                	textView.setTag(llistaOpcions.get(position)[opcions.CODI]);
//                	return textView;
//                }
          }
    }

   
    //Classe per tractar els elements de la llista de propostes
	public class PropostesAdapter extends CursorAdapter {
		 
		private final Activity context;
	    private Cursor c;
	    private LayoutInflater mInflater=null;
	         
	 
	    public PropostesAdapter(Context context, Cursor c) {
	    	super(context, c);
	    	this.context = (Activity) context;
	        this.c = c;
	        c.moveToFirst();
	        mInflater = LayoutInflater.from(context);
	    }
	    
	    
	    @Override
	    public void bindView(View view, Context context, final Cursor cursor) {
	    }
	    
	    @Override
	    public View newView(Context context, Cursor cursor, ViewGroup parent) {
	        return mInflater.inflate(R.layout.row_ocell_cercador, null);
	    }
	    
	    public View getView(int position, View convertView, ViewGroup parent) {
	    	if (convertView == null) {
	    	      mInflater = context.getLayoutInflater();
	    	      convertView = mInflater.inflate(R.layout.row_ocell_cercador, null);
	    	}   

	    	c.moveToPosition(position);
	        TextView nomCatala = (TextView)convertView.findViewById(R.id.row_ocell_cercador_nom_catala);
	        TextView nomCientific = (TextView)convertView.findViewById(R.id.row_ocell_cercador_nom_cientific);
	        ImageView imatge = (ImageView)convertView.findViewById(R.id.row_ocell_cercador_icona);
	        Button botoAfegir = (Button)convertView.findViewById(R.id.row_ocell_cercador_boto_afegir);
	               
	        String sNomCatala = c.getString(c.getColumnIndex(dbOcellsAdapter.KEY_NOM_CATALA));
			String tipus = c.getString(c.getColumnIndex(dbDadesOcellAdapter.KEY_MASCLE));
			tipus = tipus.trim() + " " + c.getString(c.getColumnIndex(dbDadesOcellAdapter.KEY_FEMELLA)).trim();
			tipus = tipus.trim() + " " + c.getString(c.getColumnIndex(dbDadesOcellAdapter.KEY_ADULT)).trim();
			tipus = tipus.trim() + " " + c.getString(c.getColumnIndex(dbDadesOcellAdapter.KEY_JOVE)).trim();
			tipus = tipus.trim() + " " + c.getString(c.getColumnIndex(dbDadesOcellAdapter.KEY_NUPCIAL)).trim();
			tipus = tipus.trim() + " " + c.getString(c.getColumnIndex(dbDadesOcellAdapter.KEY_RESTA)).trim();
	        if (!tipus.trim().equals(""))
	        	sNomCatala += " (" + tipus.trim() + ")";
	        nomCatala.setText(sNomCatala);
	        nomCientific.setText(c.getString(c.getColumnIndex(dbOcellsAdapter.KEY_NOM_CIENTIFIC)));
	        
	        String nom_imatge = c.getString(c.getColumnIndex(dbOcellsAdapter.KEY_NOM_CIENTIFIC));
	        nom_imatge = eines.nomFitxer(nom_imatge);
	        int id_imatge = context.getResources().getIdentifier(nom_imatge, "drawable", context.getPackageName());
	        
	        if (id_imatge == 0)
	        	imatge.setImageDrawable(context.getResources().getDrawable(R.drawable.icona_ocell));	
	        else
	        	imatge.setImageDrawable(context.getResources().getDrawable(id_imatge));

	        //posem colors de fons i clicable segons lo bona que es la proposta
	        Integer punts = c.getInt(c.getColumnIndex(dbDadesOcellAdapter.KEY_PUNTS));
	        if (punts >= horitzoSuperior)
	        	convertView.setBackgroundResource(R.drawable.row_propostes_millors);
	        else
	        	convertView.setBackgroundResource(R.drawable.row_propostes_pitjors);
	        
	        
	        botoAfegir.setTag(c.getInt(0));
	        botoAfegir.setOnClickListener(new OnClickListener() {
	          @Override
	          public void onClick(View v) {
	        	  final int idOcell = (Integer) v.getTag();
	        	  Intent retornIntent = new Intent(getApplicationContext(), LlistaObservacioActivity.class);
	          	  retornIntent.putExtra("id",String.valueOf(idOcell));
	          	  setResult(RESULT_OK,retornIntent);
	              finish();
	          }
	      });
	        
		    return convertView; 
	    }
	    
	}
	
    
   
}
