package catalunya.de.ocells;


import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnKeyListener;
import android.content.Intent;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.text.InputType;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;
import catalunya.de.ocells.db.dbLlistesAdapter;
import catalunya.de.ocells.db.dbOcellsAdapter;
import catalunya.de.ocells.db.dbOcellsLlistaAdapter;

public class LlistaObservacioActivity extends Activity {
	private static final Integer ACTIVITAT_CERCA_REQUEST_CODE = 1;
	public static final String PREFERENCIA_LLISTA_OBSERVACIO_ORDRE = "llista_observacio_prefs_ordre_llistes";
	private dbLlistesAdapter mdbLlistesAdapter;
	private dbOcellsAdapter mdbOcellsAdapter;
	private dbOcellsLlistaAdapter mdbOcellsLlistaAdapter;
	private LlistaObservacioAdapter mLlistaObservacioAdapter;
	private ListView lvLlistat;
	private long idLlista;
	private long idOcellDialegs;
	 
	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
	    super.onCreate(savedInstanceState);
	    setContentView(R.layout.activity_llista_observacio);
	    if (android.os.Build.VERSION.SDK_INT >= 11)
	    	getActionBar().setDisplayHomeAsUpEnabled(true);
	    lvLlistat = (ListView)findViewById(R.id.llista_observacio_llista);
	            
	    //Registrem els menu contextual
	    registerForContextMenu(lvLlistat);
	    idLlista = getIntent().getLongExtra("id", 0) ;
	    //Si s'ha creat l'activitat per compartir una llista, la compartim i destruim l'activitat
	    boolean compartir = getIntent().getBooleanExtra("share", false) ;
	    if (compartir){
	    	compartirLlista(idLlista);
	    	finish();
	    }
	    //Si creem l'activitat amb un primer ocell, l'afegim a la llista
	    long id_ocell_nou = getIntent().getLongExtra("id_ocell_nou", 0) ;
	    if (id_ocell_nou != 0){
	    	afegirOcellLlista(idLlista,id_ocell_nou,1);
	    	Toast.makeText(this, R.string.llista_observacio_toast_nova_llista_creada , Toast.LENGTH_SHORT).show();
	    }
	    carregaHead(idLlista);
	    carregaLlista(idLlista);
	    	    
	    lvLlistat.setOnItemClickListener(new OnItemClickListener() {
	          @Override
	          public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
	        	  Intent fitxaIntent = new Intent(getApplicationContext(), FitxaOcellActivity.class);
	              fitxaIntent.putExtra("id", id);
	              startActivity(fitxaIntent);
	          }
	    });
    
	}
	
	
	private void carregaLlista(long id_llista) {
		SharedPreferences prefs = getPreferences(MODE_PRIVATE); 
		int ordre = prefs.getInt(PREFERENCIA_LLISTA_OBSERVACIO_ORDRE, -1);
		mdbOcellsAdapter = new dbOcellsAdapter(this);
		mdbOcellsAdapter.open();
		Cursor cursor = mdbOcellsAdapter.getOcellsLlista(id_llista, ordre);
		
		mLlistaObservacioAdapter = new LlistaObservacioAdapter(this, cursor, idLlista);
		lvLlistat.setAdapter(mLlistaObservacioAdapter);
	    mdbOcellsAdapter.close();   
	  }
	
	  
	private void carregaHead(long id_llista) {
		   
		  mdbLlistesAdapter = new dbLlistesAdapter(this);
		  mdbLlistesAdapter.openLectura();
		  String cadenaNombre;
		  Cursor cursor = mdbLlistesAdapter.getLlista(id_llista);
	     
		  cursor.moveToFirst();
		  this.setTitle(cursor.getString(cursor.getColumnIndex(dbLlistesAdapter.KEY_NOM)));
		  String dataHora = cursor.getString(cursor.getColumnIndex(dbLlistesAdapter.KEY_DATA_HORA_CREACIO));
		  String dataHoraSimple = eines.convertirDataHoraRelativa(dataHora);
		  Integer nombre = Integer.valueOf(cursor.getString(cursor.getColumnIndex("num_ocells")));
		  if (nombre == 0) cadenaNombre = getResources().getString(R.string.llistes_nombre_ocells_en_llista_buida);
		  else cadenaNombre = getResources().getQuantityString(R.plurals.llistes_nombre_ocells_en_llista, nombre, new Object[] {nombre});
          
		  if (android.os.Build.VERSION.SDK_INT >= 11){
			  ActionBar ab = getActionBar();
			  ab.setSubtitle(dataHoraSimple + " - " + cadenaNombre);
		  }
		  mdbLlistesAdapter.close();
	}
	
	private void esborrarLlista(long id_llista){
  	  mdbLlistesAdapter = new dbLlistesAdapter(this);
  	  mdbLlistesAdapter.open();
  	  mdbLlistesAdapter.esborrarLlista(id_llista);
  	  mdbLlistesAdapter.close();
	}
	
    private void canviarNomLlista(long id_llista, String nom_nou){
  	  mdbLlistesAdapter = new dbLlistesAdapter(this);
  	  mdbLlistesAdapter.open();
  	  mdbLlistesAdapter.setNomLlista(id_llista, nom_nou);
  	  mdbLlistesAdapter.close();
    }
    
    public void canviarNombreOcellsLlista(long id_llista, long id_ocell, int nombre){
      mdbOcellsLlistaAdapter = new dbOcellsLlistaAdapter(this);
      mdbOcellsLlistaAdapter.open();
      mdbOcellsLlistaAdapter.setNombreOcellsLlista(id_llista, id_ocell, nombre);
      mdbLlistesAdapter.close();
    }
    
    private void setAnotacioOcell(long id_llista, long id_ocell, String anotacio){
  	  mdbOcellsLlistaAdapter = new dbOcellsLlistaAdapter(this);
  	  mdbOcellsLlistaAdapter.open();
  	  mdbOcellsLlistaAdapter.setAnotacioOcell(id_llista, id_ocell, anotacio);
  	  mdbLlistesAdapter.close();
    }
    
    private void afegirOcellLlista(long id_llista, long id_ocell, int nombre){
    	mdbOcellsLlistaAdapter = new dbOcellsLlistaAdapter(this);
 		mdbOcellsLlistaAdapter.open();
 		mdbOcellsLlistaAdapter.afegirOcellLlista(id_llista,id_ocell,nombre);
 		mdbOcellsLlistaAdapter.close();
    }
    
	protected void onActivityResult(int request_code, int result_code, Intent data) {
		super.onActivityResult(request_code, result_code, data);
		if (request_code == ACTIVITAT_CERCA_REQUEST_CODE ) {
		     if(result_code == RESULT_OK){
		    	 if (data.getExtras().containsKey("id")){
		    		 afegirOcellLlista(idLlista,Long.valueOf(data.getStringExtra("id")),1);
		    		 carregaHead(idLlista);
		    		 carregaLlista(idLlista);
		    	 }
		     } else if (result_code == RESULT_CANCELED) {
		 			//Write your code on no result return 
		     }
		}

		super.onActivityResult(request_code, result_code, data);
	}	
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.activity_llista_observacio, menu);
		return true;
	}
	
	@Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        	case android.R.id.home:
        		finish();
        		return true;
	        case R.id.llista_observacio_menu_cerca_escrita:
	        	final Intent cercaIntent = new Intent(getApplicationContext(),	CercaEscritaActivity.class);  
	    		startActivityForResult(cercaIntent, ACTIVITAT_CERCA_REQUEST_CODE);
	    		return true;
	        case R.id.llista_observacio_menu_cerca_inteligent:
	        	final Intent cercaIntent2 = new Intent(getApplicationContext(),	CercaInteligentActivity.class);  
	    		startActivityForResult(cercaIntent2, ACTIVITAT_CERCA_REQUEST_CODE);
	    		return true;
	        case R.id.llista_observacio_menu_dictar_veu:
        		Intent fitxaIntent = new Intent(getApplicationContext(), CercaReconeixementVeuActivity.class);
                startActivityForResult(fitxaIntent, ACTIVITAT_CERCA_REQUEST_CODE);
                return true;
	        case R.id.llista_observacio_menu_copiar:
	        	mdbLlistesAdapter = new dbLlistesAdapter(this);
		    	mdbLlistesAdapter.open();
		    	long id_nova_llista = mdbLlistesAdapter.copiarLlista(idLlista, getString(R.string.llistes_nom_copia_llista));
		    	//idLlista = id_nova_llista;
		    	mdbLlistesAdapter.close();
		    	Intent LlistaIntent2 = new Intent(this, LlistaObservacioActivity.class);
	            LlistaIntent2.putExtra("id", id_nova_llista);
	            startActivity(LlistaIntent2);
	            Toast.makeText(this, R.string.llista_observacio_toast_nova_llista_creada , Toast.LENGTH_SHORT).show();
				return true;
	        case R.id.llista_observacio_menu_compartir:
        		compartirLlista(idLlista);
        		return true;
	        case R.id.llista_observacio_menu_ordenar:
        		crearDialegCanviOrdre();
        		return true;
        	case R.id.llista_observacio_menu_canviar_nom:
	    		crearDialegCanviNomLlista();
	    		return true;
        	case R.id.llista_observacio_menu_esborrar:
        		crearDialegConfirmacioEsborrarLlista();
	    		return true;
        	case R.id.llista_observacio_menu_preferencies:
           	 	Intent intent = new Intent(getApplicationContext(), SettingsActivity.class);
                startActivity(intent);            	
                return true;
        	case R.id.llista_observacio_menu_ajuda:
 	        	Intent i = new Intent(Intent.ACTION_VIEW);
 	           	i.setData(Uri.parse(this.getResources().getString((R.string.url_ajuda))));
 	           	startActivity(i);           	
 	           	return true;
            default:
                return false;
        }
	}	
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View v,ContextMenuInfo menuInfo){
		super.onCreateContextMenu(menu, v, menuInfo);
		MenuInflater inflater = getMenuInflater();
		if(v.getId() == R.id.llista_observacio_llista){
			//AdapterView.AdapterContextMenuInfo info =(AdapterView.AdapterContextMenuInfo)menuInfo;
			menu.setHeaderTitle(R.string.llista_observacio_menu_ctx_ocell_titol);
			inflater.inflate(R.menu.activity_llista_observacio_menu_ctx_ocell, menu);
		}

	}
	
	@Override
	public boolean onContextItemSelected(MenuItem item) {
		AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
		switch (item.getItemId()) {	
			case R.id.llista_observacio_menu_ctx_ocell_veure:
			    Intent fitxaIntent = new Intent(getApplicationContext(), FitxaOcellActivity.class);
                fitxaIntent.putExtra("id", info.id);
                startActivity(fitxaIntent);
				return true;
			case R.id.llista_observacio_menu_ctx_ocell_anotacio:
				idOcellDialegs = info.id;
				crearDialegAnotacio(info.id);
				return true;
			case R.id.llista_observacio_menu_ctx_ocell_esborrar:
				 mdbOcellsLlistaAdapter = new dbOcellsLlistaAdapter(this);
	    		 mdbOcellsLlistaAdapter.open();
	    		 mdbOcellsLlistaAdapter.esborraOcellLlista(idLlista,info.id);
	    		 mdbOcellsLlistaAdapter.close();
	    		 carregaHead(idLlista);
	    		 carregaLlista(idLlista);
				 return true;
			default:
				return super.onContextItemSelected(item);
		}
	}
	
	private void crearDialegConfirmacioEsborrarLlista()
    {
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	builder.setTitle(R.string.llistes_dialeg_confirmacio_esborrar_llista_titol);
    	builder.setMessage(R.string.llistes_dialeg_confirmacio_esborrar_llista_descripcio);
    	builder.setPositiveButton(R.string.llistes_dialeg_confirmacio_esborrar_llista_afirmacio, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				esborrarLlista(idLlista);
				//dialog.cancel();
				finish();
			}
		});
    	builder.setNegativeButton(R.string.llistes_dialeg_confirmacio_esborrar_llista_negacio, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.cancel();
			}
		});
    	builder.show();
    	
    }
	
	private void crearDialegAnotacio(long id_ocell)
    {
		final EditText input = new EditText(this);
    	mdbOcellsLlistaAdapter = new dbOcellsLlistaAdapter(this);
    	mdbOcellsLlistaAdapter.open();
      	input.setText(mdbOcellsLlistaAdapter.getAnotacioOcell(idLlista, id_ocell));
		final AlertDialog.Builder dialog = new AlertDialog.Builder(this);
		dialog.setTitle(R.string.llista_observacio_dialeg_anotacio_ocell_titol);
    	dialog.setView(input);
    	dialog.setPositiveButton(R.string.llista_observacio_dialeg_anotacio_ocell_desar, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				setAnotacioOcell(idLlista, idOcellDialegs, input.getText().toString());
				carregaHead(idLlista);
				mLlistaObservacioAdapter.reloadCursor();
				//carregaLlista(idLlista);
				dialog.cancel();
			}
		});
    	dialog.setNegativeButton(R.string.llista_observacio_dialeg_anotacio_ocell_cancelar, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.cancel();
			}
		});
    	   	
    	dialog.show();	
    	input.requestFocus();
    	input.setFocusable(true);
    	input.requestFocus();
    	// TODO No va l'aparicio del teclat automatica
    	//InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        //imm.toggleSoftInput(InputMethodManager.SHOW_FORCED, 0);
    	mdbOcellsLlistaAdapter.close();
    }
	

	
	private void crearDialegCanviNomLlista()
    {
		final EditText input = new EditText(this);
    	mdbLlistesAdapter = new dbLlistesAdapter(this);
      	mdbLlistesAdapter.open();
      	input.setText(mdbLlistesAdapter.getNomLlista(idLlista));
      	input.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_FLAG_IME_MULTI_LINE);
		final AlertDialog.Builder dialog = new AlertDialog.Builder(this);
		dialog.setTitle(R.string.llista_observacio_dialeg_canvi_nom_titol);
    	dialog.setView(input);
    	dialog.setPositiveButton(R.string.llista_observacio_dialeg_canvi_nom_desar, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				canviarNomLlista(idLlista, input.getText().toString());
				carregaHead(idLlista);
				dialog.dismiss();
			}
		});
    	dialog.setNegativeButton(R.string.llista_observacio_dialeg_canvi_nom_cancelar, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.cancel();
			}
		});
    	dialog.setOnKeyListener(new OnKeyListener() {
      	    public boolean onKey(DialogInterface d, int keyCode, KeyEvent event) {
      	        if ((event.getAction() == KeyEvent.ACTION_DOWN) && (keyCode == KeyEvent.KEYCODE_ENTER)) {
      	        	canviarNomLlista(idLlista, input.getText().toString());
    				carregaHead(idLlista);
    				d.dismiss();
      	          	return true;
      	        }
      	        return false;
      	    }
      	});
    	dialog.show();	
    	input.requestFocus();
    	input.selectAll();
    	mdbLlistesAdapter.close();
    }
	
	private void crearDialegCanviOrdre(){
		SharedPreferences prefs = getPreferences(MODE_PRIVATE); 
		int ordre = prefs.getInt(PREFERENCIA_LLISTA_OBSERVACIO_ORDRE, -1);
		final String[] items = {
				 getResources().getString(R.string.llista_observacio_dialeg_canvi_ordre_nom_catala),
				 getResources().getString(R.string.llista_observacio_dialeg_canvi_ordre_nom_cientific),
				 getResources().getString(R.string.llista_observacio_dialeg_canvi_ordre_familia),
				 getResources().getString(R.string.llista_observacio_dialeg_canvi_ordre_insercio)};
		AlertDialog.Builder dialog = new AlertDialog.Builder(this);
		dialog.setTitle(R.string.llista_observacio_dialeg_canvi_ordre_titol);
		dialog.setSingleChoiceItems(items, ordre, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int item) { 
				SharedPreferences.Editor prefs = getPreferences(MODE_PRIVATE).edit();
				prefs.putInt(PREFERENCIA_LLISTA_OBSERVACIO_ORDRE, item);
				prefs.commit();
				dialog.dismiss();
				carregaLlista(idLlista);
			 }
		});
		dialog.show();
	}
	
	private void compartirLlista(long id_llista) {//sharing implementation here
		Intent sharingIntent = new Intent(android.content.Intent.ACTION_SEND);
		sharingIntent.setType("text/plain");
		String compartirBody = "";
		SharedPreferences prefs = getPreferences(MODE_PRIVATE); 
		int ordre = prefs.getInt(PREFERENCIA_LLISTA_OBSERVACIO_ORDRE, -1);
		mdbOcellsAdapter = new dbOcellsAdapter(this);
		mdbOcellsAdapter.open();
		Cursor cursor = mdbOcellsAdapter.getOcellsLlista(id_llista, ordre);
		for (int i = 0; i < cursor.getCount();i++){
			cursor.moveToPosition(i);
			compartirBody += System.getProperty("line.separator");
			compartirBody += cursor.getString(cursor.getColumnIndex(dbOcellsLlistaAdapter.KEY_QUANTITAT)) + ": ";
			compartirBody += cursor.getString(cursor.getColumnIndex(dbOcellsAdapter.KEY_NOM_CATALA)) + " (";
			compartirBody += cursor.getString(cursor.getColumnIndex(dbOcellsAdapter.KEY_NOM_CIENTIFIC)) + ")";
			String anotacio = cursor.getString(cursor.getColumnIndex(dbOcellsLlistaAdapter.KEY_ANOTACIO));
			if (anotacio!=null) compartirBody += " - " + anotacio;
		}
		
		sharingIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, compartirCrearAssumpteLlista(idLlista));
		sharingIntent.putExtra(android.content.Intent.EXTRA_TEXT, compartirBody);
		startActivity(Intent.createChooser(sharingIntent, getResources().getString(R.string.llista_observacio_missatge_compartir)));
		mdbOcellsAdapter.close();
	}
		
	private String compartirCrearAssumpteLlista(long id_llista){
		mdbLlistesAdapter = new dbLlistesAdapter(this);
		mdbLlistesAdapter.open();
		Cursor cursor = mdbLlistesAdapter.getLlista(id_llista);
		cursor.moveToFirst();
	    String assumpte;
		assumpte = cursor.getString(cursor.getColumnIndex(dbLlistesAdapter.KEY_NOM)) + " - ";
	    int nombre = Integer.valueOf(cursor.getString(cursor.getColumnIndex("num_ocells"))); //Camp calculat per BBDD
	    String cadenaNombre = getResources().getQuantityString(R.plurals.llistes_nombre_ocells_en_llista, nombre, new Object[] {nombre});
	    if (nombre == 0) assumpte += getResources().getString(R.string.llistes_nombre_ocells_en_llista_buida);
	    else assumpte += cadenaNombre;
	    assumpte += " - " + eines.convertirDataHoraSimple(cursor.getString(cursor.getColumnIndex(dbLlistesAdapter.KEY_DATA_HORA_CREACIO)));
	    mdbLlistesAdapter.close();
	    return assumpte;
	}
	 

}
