package catalunya.de.ocells;

import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnKeyListener;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.text.InputType;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.Toast;
import android.widget.SimpleCursorAdapter.ViewBinder;
import android.widget.TextView;
import catalunya.de.ocells.db.dbLlistesAdapter;
import catalunya.de.ocells.db.dbOcellsLlistaAdapter;

public class LlistesActivity extends Activity {
 private static final Integer ACTIVITAT_CERCA_REQUEST_CODE = 1;
 private static final String LLISTES_PREFERENCIA_ORDRE = "llistes_activity_ordre_especies";
 private dbLlistesAdapter mdbLlistesAdapter;
 private ListView lvLlistat;
 private long idLlistaDialegs;


  /** Called when the activity is first created. */
  @Override
  public void onCreate(Bundle savedInstanceState) {
      super.onCreate(savedInstanceState);
      setContentView(R.layout.activity_llistes);
      lvLlistat = (ListView)findViewById(R.id.llistes_llista);
      //Afegim el menu contextual
      registerForContextMenu(lvLlistat);

      lvLlistat.setOnItemClickListener(new OnItemClickListener() {
          @Override
          public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        	  Intent LlistaIntent = new Intent(getApplicationContext(), LlistaObservacioActivity.class);
              LlistaIntent.putExtra("id", id);
              startActivity(LlistaIntent);
          }
      });
  }
  
  
  
  @Override
  public void onResume() {
	  super.onResume();
	  carregaLlistes();
	  mdbLlistesAdapter = new dbLlistesAdapter(this);
	  mdbLlistesAdapter.open();
	  
	  if (!mdbLlistesAdapter.taulaOcellsPlena()){
		  Intent LlistaIntent = new Intent(getApplicationContext(), HolaActivity.class);
          startActivity(LlistaIntent);
	  }
	  mdbLlistesAdapter.close();//////////
	  
  }
  
  private int carregaLlistes() {
	  SharedPreferences prefs = getPreferences(MODE_PRIVATE); 
	  int ordre = prefs.getInt(LLISTES_PREFERENCIA_ORDRE, -1);
	  mdbLlistesAdapter = new dbLlistesAdapter(this);
	  mdbLlistesAdapter.openLectura();
	  Cursor cursor = mdbLlistesAdapter.llistaTot(ordre);
     
	  String[] from = new String[]{
   		dbLlistesAdapter.KEY_NOM, 
   		dbLlistesAdapter.KEY_DATA_HORA_CREACIO,
   		"num_ocells"};
	  int[] to = new int[]{
   		  R.id.row_llistes_nom, 
   		  R.id.row_llistes_data_hora,
   		  R.id.row_llistes_nombre };

	  SimpleCursorAdapter cursorAdapter =  new SimpleCursorAdapter(this, R.layout.row_llistes, cursor, from, to);
	  //Modifiquem els resultats del cursor abans de carregar-lo a la llista
	  cursorAdapter.setViewBinder(new ViewBinder() {
		    public boolean setViewValue(View aView, Cursor aCursor, int aColumnIndex) {
		        if (aColumnIndex == 3) {
	                Integer nombre = Integer.valueOf(aCursor.getString(3));
	                String cadenaNombre = getResources().getQuantityString(R.plurals.llistes_nombre_ocells_en_llista, nombre, new Object[] {nombre});
	                TextView textView = (TextView) aView;
	                if (nombre == 0) textView.setText(R.string.llistes_nombre_ocells_en_llista_buida);
	                else textView.setText(cadenaNombre);
	                return true;
		         }
		        if (aColumnIndex == 2) { 
		        	String datahora = aCursor.getString(2);
		        	TextView textView = (TextView) aView;
		        	textView.setText(eines.convertirDataHoraRelativa(datahora));
		        	return true;
		        }
		        return false;
		    }
		});
	      
	  lvLlistat.setAdapter(cursorAdapter);
	  int n = cursor.getCount();
	  String countString = getResources().getQuantityString(R.plurals.llistes_nombre_llistes,
              n, new Object[] {n});
	  if (android.os.Build.VERSION.SDK_INT >= 11){
		  ActionBar ab = getActionBar();
		  ab.setSubtitle(countString) ;
	  }
	  
	  mdbLlistesAdapter.close();
	  return (n);
	
  }
  
    private long crearLlista(){
    	mdbLlistesAdapter = new dbLlistesAdapter(this);
    	mdbLlistesAdapter.open();
    	long id_nova_llista = mdbLlistesAdapter.crearLlista(getString(R.string.llistes_nom_nova_llista), true);
    	mdbLlistesAdapter.close();
    	return id_nova_llista;
    }
    
    private void esborrarLlista(long id_llista){
  	  	mdbLlistesAdapter = new dbLlistesAdapter(this);
  	  	mdbLlistesAdapter.open();
  	  	mdbLlistesAdapter.esborrarLlista(id_llista);
  	  	mdbLlistesAdapter.close();
    }
    
    private void canviarNomLlista(long id_llista, String nom_nou){
    	mdbLlistesAdapter = new dbLlistesAdapter(this);
    	mdbLlistesAdapter.open();
    	mdbLlistesAdapter.setNomLlista(id_llista, nom_nou);
    	mdbLlistesAdapter.close();
    }

	protected void onActivityResult(int request_code, int result_code, Intent data) {
		super.onActivityResult(request_code, result_code, data);
		if (request_code == ACTIVITAT_CERCA_REQUEST_CODE ) {
		     if(result_code == RESULT_OK){
		    	 if (data.getExtras().containsKey("id")){
		    		 long id_nova_llista = crearLlista();
		    		 Intent LlistaIntent = new Intent(getApplicationContext(), LlistaObservacioActivity.class);
	                 LlistaIntent.putExtra("id", id_nova_llista);
	                 LlistaIntent.putExtra("id_ocell_nou", Long.valueOf(data.getStringExtra("id")));
	                 startActivity(LlistaIntent);
		    	 }
		     } else if (result_code == RESULT_CANCELED) {
		 			//Write your code on no result return 
		     }
		}

		super.onActivityResult(request_code, result_code, data);
	}	
    
  	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.activity_llistes, menu);
		return true;
	}
	
	@Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.llistes_menu_nova_llista:
            	long id_nova_llista = crearLlista();
             	Intent LlistaIntent = new Intent(getApplicationContext(), LlistaObservacioActivity.class);
                LlistaIntent.putExtra("id", id_nova_llista);
                startActivity(LlistaIntent);
                return true;
            case R.id.llistes_menu_cerca_escrita:
	        	final Intent cercaIntent = new Intent(getApplicationContext(),	CercaEscritaActivity.class);  
	    		startActivityForResult(cercaIntent, ACTIVITAT_CERCA_REQUEST_CODE);
	    		return true;
	        case R.id.llistes_menu_cerca_inteligent:
	        	final Intent cercaIntent2 = new Intent(getApplicationContext(),	CercaInteligentActivity.class);  
	    		startActivityForResult(cercaIntent2, ACTIVITAT_CERCA_REQUEST_CODE);
	    		return true;
            case R.id.llistes_menu_ordenar:
            	crearDialegCanviOrdre();
            	return true;
            case R.id.llistes_menu_preferencies:
            	Intent intent = new Intent(getApplicationContext(), SettingsActivity.class);
                startActivity(intent);            	
            	return true;
            case R.id.llistes_menu_ajuda:
	        	Intent i = new Intent(Intent.ACTION_VIEW);
	           	i.setData(Uri.parse(this.getResources().getString((R.string.url_ajuda))));
	           	startActivity(i);           	
	           	return true;
            default:
                return false;
        }
	}
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View v,
			ContextMenuInfo menuInfo){
		super.onCreateContextMenu(menu, v, menuInfo);
		MenuInflater inflater = getMenuInflater();
		if(v.getId() == R.id.llistes_llista){
			menu.setHeaderTitle(R.string.llistes_menu_ctx_llista_titol);
			inflater.inflate(R.menu.activity_llistes_menu_ctx_llista, menu);
		}
	}
	
	@Override
	public boolean onContextItemSelected(MenuItem item) {
		AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
		switch (item.getItemId()) {	
			case R.id.llistes_menu_ctx_llista_veure:
				Intent LlistaIntent = new Intent(getApplicationContext(), LlistaObservacioActivity.class);
	            LlistaIntent.putExtra("id", info.id);
	            startActivity(LlistaIntent);
				return true;
			case R.id.llistes_menu_ctx_llista_canviar_nom:
				idLlistaDialegs = info.id;
				crearDialegCanviNomLlista();
				return true;
			case R.id.llistes_menu_ctx_llista_compartir:
				Intent LlistaIntent2 = new Intent(getApplicationContext(), LlistaObservacioActivity.class);
	            LlistaIntent2.putExtra("id", info.id);
	            LlistaIntent2.putExtra("share", true);
	            startActivity(LlistaIntent2);
				return true;
			case R.id.llistes_menu_ctx_llista_copiar:
				mdbLlistesAdapter = new dbLlistesAdapter(this);
		    	mdbLlistesAdapter.open();
		    	long id_nova_llista = mdbLlistesAdapter.copiarLlista(info.id, getString(R.string.llistes_nom_copia_llista));
		    	Intent LlistaIntent3 = new Intent(getApplicationContext(), LlistaObservacioActivity.class);
	            LlistaIntent3.putExtra("id", id_nova_llista);
	            startActivity(LlistaIntent3);
	            Toast.makeText(this, R.string.llista_observacio_toast_nova_llista_creada , Toast.LENGTH_SHORT).show();
				mdbLlistesAdapter.close();
				return true;
			case R.id.llistes_menu_ctx_llista_esborrar:
				idLlistaDialegs = info.id;
				crearDialegConfirmacioEsborrarLlista();
				return true;
			default:
				return super.onContextItemSelected(item);
		}
	}

	
	private void crearDialegConfirmacioEsborrarLlista()
    {
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	builder.setTitle(R.string.llistes_dialeg_confirmacio_esborrar_llista_titol);
    	builder.setMessage(R.string.llistes_dialeg_confirmacio_esborrar_llista_descripcio);
    	builder.setPositiveButton(R.string.llistes_dialeg_confirmacio_esborrar_llista_afirmacio, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				esborrarLlista(idLlistaDialegs);
				carregaLlistes();
				dialog.cancel();
			}
		});
    	builder.setNegativeButton(R.string.llistes_dialeg_confirmacio_esborrar_llista_negacio, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.cancel();
			}
		});
    	builder.show();
    	
    }
	
	
	private void crearDialegCanviNomLlista()
    {
		final EditText input = new EditText(this);
    	mdbLlistesAdapter = new dbLlistesAdapter(this);
      	mdbLlistesAdapter.open();
      	input.setText(mdbLlistesAdapter.getNomLlista(idLlistaDialegs));
      	input.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_FLAG_IME_MULTI_LINE);
		final AlertDialog.Builder dialog = new AlertDialog.Builder(this);
		dialog.setTitle(R.string.llistes_dialeg_canvi_nom_titol);
    	dialog.setView(input);
    	dialog.setPositiveButton(R.string.llistes_dialeg_canvi_nom_desar, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				canviarNomLlista(idLlistaDialegs, input.getText().toString());
				carregaLlistes();
				dialog.dismiss();
			}
		});
    	dialog.setNegativeButton(R.string.llistes_dialeg_canvi_nom_cancelar, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.cancel();
			}
		});
    	dialog.setOnKeyListener(new OnKeyListener() {
      	    public boolean onKey(DialogInterface d, int keyCode, KeyEvent event) {
      	        if ((event.getAction() == KeyEvent.ACTION_DOWN) && (keyCode == KeyEvent.KEYCODE_ENTER)) {
      				canviarNomLlista(idLlistaDialegs, input.getText().toString());
      				carregaLlistes();
    				d.dismiss();
      	          	return true;
      	        }
      	        return false;
      	    }
      	});
    	dialog.show();	
    	input.requestFocus();
    	input.selectAll();
    	mdbLlistesAdapter.close();
    }
	
	private void crearDialegCanviOrdre(){
		SharedPreferences prefs = getPreferences(MODE_PRIVATE); 
		int ordre = prefs.getInt(LLISTES_PREFERENCIA_ORDRE, -1);
		final String[] items = {
				 getResources().getString(R.string.llistes_dialeg_canvi_ordre_nom),
				 getResources().getString(R.string.llistes_dialeg_canvi_ordre_data),
				 getResources().getString(R.string.llistes_dialeg_canvi_ordre_data_desc)};
		AlertDialog.Builder dialog = new AlertDialog.Builder(this);
		dialog.setTitle(R.string.llistes_dialeg_canvi_ordre_titol);
		dialog.setSingleChoiceItems(items, ordre, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int item) { 
				SharedPreferences.Editor editor = getPreferences(MODE_PRIVATE).edit();
				editor.putInt(LLISTES_PREFERENCIA_ORDRE, item);
			    editor.commit();
				dialog.dismiss();
				carregaLlistes();
			 }
		});
		dialog.show();	
	}
	
}