package catalunya.de.ocells.cerca;

import java.util.ArrayList;

import android.content.Context;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.preference.PreferenceManager;
import android.util.Log;
import android.widget.Toast;
import catalunya.de.ocells.R;
import catalunya.de.ocells.SettingsActivity;
import catalunya.de.ocells.eines;
import catalunya.de.ocells.db.dbDadesOcellAdapter;
/**
 * Classe preguntes
 * @author jsirera
 *
 */
public class preguntes {

	private Context mContext;
	private dbDadesOcellAdapter mdbDadesOcellAdapter;
	public ArrayList<pregunta> p;
	public ArrayList<opcions> o;
	public Integer preguntesFetes;
	public Integer TotalPuntsSumats;
		
	
	public preguntes(Context context) {
		mContext = context;
		p = defs.carregaPreguntes();
		o = defs.carregaOpcions();
		preguntesFetes = 0; 
		TotalPuntsSumats = defs.PUNTS_INICIALS;
    }
	
	public void reiniciarPunts(){
		//Reiniciem puntuacions de tots els ocells
		mdbDadesOcellAdapter = new dbDadesOcellAdapter(mContext);
		mdbDadesOcellAdapter.open();
		mdbDadesOcellAdapter.reiniciarPuntuacions(); 
		mdbDadesOcellAdapter.close();
	}
	
	
	public void aplicarCaracteristiquesImplicites(){
		//Obtenim les preferencies per saber si s'ha d'aplicar la fenologia i la raresa
		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(mContext);
		boolean usarFenologia = pref.getBoolean("usar_fenologia", true);
		boolean usarRaresa = pref.getBoolean("usar_raresa", true);
		
		//Apliquem caracteristiques implicites
		if (usarFenologia)
			aplicarFenologia();
		if (usarRaresa)
			aplicarRaresa();
			
		p.get(getIdPregunta("fenologia")).bFeta = true;
		p.get(getIdPregunta("rar")).bFeta = true;
	}
	
	
	public void aplicarFenologia(){
		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(mContext);
		String dataFenologia = pref.getString("preferencies_data_fenologia", "-1");

		if (dataFenologia.equals("-1"))
			p.get(getIdPregunta("fenologia")).keyDB = "fenologia" + eines.getMesNumeric();
		else
			p.get(getIdPregunta("fenologia")).keyDB = "fenologia" + dataFenologia;
		actualitzaProbabilitats(getIdPregunta("fenologia"),"2");
	}
	
	public void aplicarRaresa(){
		actualitzaProbabilitats(getIdPregunta("rar"),"");
	}
	
	/**
	 * Retorna el id de la millor pregunta a fer per discriminar mes espcies
	 * @return id de la millor pregunta
	 */
	public Integer cercarPreguntaOptima(){
		
		//Si es la primera, preguntarem per l'habitat
		if (preguntesFetes==0) return getIdPregunta("habitat"); 
		
		//Si no s'esta contestant a res, introduim pregunta xorra
		Integer idPreguntaXorra = getIdPregunta("preguntaXorra");
		if (preguntesFetes > 10 && TotalPuntsSumats < 1000 && !p.get(idPreguntaXorra).bFeta) return idPreguntaXorra; 
		
		//Si tenim 3 especies a dalt i es ja permet aquest tipus e preguntes, pregunta per la prioritat
		Integer idPregunta = preguntaPrioritat();
		if (idPregunta != defs.NO) 
			return idPregunta;

		//Si el primer ocell t caracterstica nica i ja es permeten aquestes preguntes
		if (preguntaCaracteristicaUnica()) 
			return getIdPregunta("caracteristicaUnica");

		//Seleccionem espcies que es desmarquin amb puntuaci
		//Aix ho farem cercant la puntuaci que les divideixi en dos grups per a cada horitzo
		Integer puntsHoritzoSuperior = cercarHoritzo(defs.PERCENT_HORITZO_SUPERIOR);
		Integer puntsHoritzoInferior = cercarHoritzo(defs.PERCENT_HORITZO_INFERIOR);

		//Cerquem la caracterstica que diferenca mes a les espcies per damunt de l'horitzo de divisi
		//Per fer-ho cerquem la caracterstica amb menor desviaci estandard.
		mdbDadesOcellAdapter = new dbDadesOcellAdapter(mContext);
		mdbDadesOcellAdapter.open();		
		double desvEstanadardMinima=defs.MOLT_ALT;
		ArrayList<Integer> valorsSuperior;
		ArrayList<Integer> valorsInferior;
		Integer idPreguntaOptima=defs.SENSE_PREGUNTES;
		
		for (int i = 0; i < p.size(); i++) {
			pregunta mP = p.get(i);
			
			if (mP.bActiva && !mP.bFeta && mP.ordreMinim<=preguntesFetes && mP.tipus==pregunta.CONCRETA){
				 valorsSuperior = mdbDadesOcellAdapter.getEspeciesPerOpcio(mP.keyDB, puntsHoritzoSuperior);
				 valorsInferior = mdbDadesOcellAdapter.getEspeciesPerOpcio(mP.keyDB, puntsHoritzoInferior);
				//Si noms hi ha una opci des de l'inferior, abandonem la pregunta:
				 if (valorsInferior.size() != 1) {
					 double desEstandard;
					 if (valorsSuperior.size() == 1) 
						 //Si hi ha noms una opcio per damunt del superior, calculem la desviaci desde l'horitzo inferior
						 //per la multipliquem per a que no compti tant com les altres preguntes
						 desEstandard = eines.desviacioEstandard(valorsInferior) * 100;
					 else
						 //Si hi ha varies opcions per damunt de l'horitzo superior, calculem la desviaci estndard.
						 desEstandard = eines.desviacioEstandard(valorsSuperior);	 
	
					 if (desEstandard < desvEstanadardMinima) {
						 desvEstanadardMinima = desEstandard;
						 idPreguntaOptima = i;
					 }
				 }
			}
		}
		mdbDadesOcellAdapter.close();
		return idPreguntaOptima;
	}
	
	/**
	 * retorna les opcions d'una pregunta amb parells <codi_opcio, recurs>
	 * @param id_pregunta id de la pregunta
	 * @return llista amb <codi_opcio, recurs>
	 */
	public ArrayList<String[]> getOpcionsPregunta(Integer id_pregunta){
		ArrayList<String[]> opcionsPerMostrar = new ArrayList<String[]>();
		Integer idOpcions = 0;
		String c, r, t;
		//Si es tracta de preguntes on s'han de mostrar totes les opcions...
		Integer tipusPregunta = p.get(id_pregunta).tipus;
		if (tipusPregunta == pregunta.HABITAT || tipusPregunta == pregunta.CARACTERISTICA || tipusPregunta == pregunta.XORRA){
			idOpcions =  getIdOpcionsPregunta(p.get(id_pregunta).opcions);
			for (int i = 0; i < o.get(idOpcions).codiOpcio.length; i++) {
				c = o.get(idOpcions).codiOpcio[i];
				r = o.get(idOpcions).recurs[i];
				t = o.get(idOpcions).text[i];
				opcionsPerMostrar.add(new String[] {c, r, t});
			}
		}
		else {
			//Si es una altra, retornem les opcions que com a mnim son en alguna espcie
			//i que aquesta especie esta per sobre de l'horitzo inferior
			String nom_opcions = p.get(id_pregunta).opcions;
			idOpcions = getIdOpcionsPregunta(nom_opcions);
			Integer horitzoInferior = cercarHoritzo(defs.PERCENT_HORITZO_INFERIOR);
			mdbDadesOcellAdapter = new dbDadesOcellAdapter(mContext);
			mdbDadesOcellAdapter.open();
			ArrayList<String> opcionsDB = mdbDadesOcellAdapter.getOpcionsUsades(p.get(id_pregunta).keyDB, horitzoInferior);
			for (int i = 0; i < opcionsDB.size(); i++) {
				Integer idOpcio = o.get(idOpcions).getIdOpcio(opcionsDB.get(i));
				c = o.get(idOpcions).codiOpcio[idOpcio];
				r = o.get(idOpcions).recurs[idOpcio];
				t = o.get(idOpcions).text[idOpcio];
				opcionsPerMostrar.add(new String[] { c, r, t } );
			}
			if (opcionsDB.size()==0)  Log.e("ocells", "preguntes.java: No s'han trobat opcions a la bbdd de: " + nom_opcions);
			mdbDadesOcellAdapter.close();
		}
		
		//Afegim la opcio "No ho veig, no ho se..." amb codi i recurs
		c = o.get(idOpcions).senseResposta[opcions.CODI];
		r = o.get(idOpcions).senseResposta[opcions.RECURS];
		t = o.get(idOpcions).senseResposta[opcions.TEXT];
		opcionsPerMostrar.add(new String[] { c, r, t } );
		return opcionsPerMostrar;
	}
	
	/**
	 * Retorna el id de les opcions d'una pregunta a partir del nom
	 * @param nom_opcions nom de l'opci
	 * @return id de l'opci
	 */
	public Integer getIdOpcionsPregunta(String nom_opcions){
		boolean trobat = false;
		Integer idOpcions = 0;
		for (int i = 0; i < o.size() && !trobat; i++) {
			if (o.get(i).nom.equals(nom_opcions)){
				trobat = true;
				idOpcions = i;
			}
		}
		if (!trobat) Log.e("ocells", "preguntes.java: No s'han trobat les opcions: " + nom_opcions + " per a la preguta!");
		return idOpcions;
	}
	
	/**
	 * Retorna el id d'una pregunta a partir del nom
	 * @param nom_pregunta nom d ela pregunta
	 * @return id de la pregunta
	 */
	public Integer getIdPregunta(String nom_pregunta){
		boolean trobat = false;
		Integer idPregunta = 0;
		for (int i = 0; i < p.size() && !trobat; i++) {
			if (p.get(i).nom.equals(nom_pregunta)){
				trobat = true;
				idPregunta = i;
			}
		}
		if (!trobat) Log.e("ocells", "preguntes.java: No s'han trobat la pregunta: " + nom_pregunta );
		return idPregunta;
	}
	
	/**
	 * Actualitza l'estat de les preguntes despres de fer-ne alguna. S'arrengen algunes exepcions de preguntes especials 
	 * @param id_pregunta id de la pregunta
	 * @param codi_opcio codi de la opci resposta
	 * @return codi opci nou en el cas de pregunta d'habitat
	 */
	public String actualitzaPreguntes(Integer id_pregunta, String codi_opcio){
		String codiOpcio = codi_opcio;
		p.get(id_pregunta).bFeta = true;
		preguntesFetes++;
		
		//Si es tracta de pregunta d'habitat tenim que actualitzar l'apuntador al camp de la BBDD correcte
		//Si s'ha contestat algun habitat canviem la resposta per '2'
		if (p.get(id_pregunta).tipus==pregunta.HABITAT){
			p.get(id_pregunta).opcions = "opcionsHabitatPerCalcul";
			if (!codi_opcio.equals(defs.SENSE_RESPOSTA) ){
				p.get(id_pregunta).keyDB = codi_opcio;
				codiOpcio = "2";
			}
		}

		//Si es la pregunta de caracteristica unica...
		if (p.get(id_pregunta).tipus==pregunta.CARACTERISTICA){
			//Si no s'ha respost, baixem el lder a segona posici
			if (codi_opcio.equals(defs.SENSE_RESPOSTA)){
				mdbDadesOcellAdapter = new dbDadesOcellAdapter(mContext);
				mdbDadesOcellAdapter.open();
					mdbDadesOcellAdapter.orAPlata();
				mdbDadesOcellAdapter.close();
				Log.e("ocells", "s'ha respost que no se sap");
				//Toast.makeText(mContext, mContext.getString(R.string.cerca_inteligent_caracteristica_unica_sense_resposta) , Toast.LENGTH_SHORT).show();
			} else if (codi_opcio.equals("si"))
				//Si contesta que si la desactivem per a que no es torni a fer
				p.get(id_pregunta).bActiva = false;
		}
		
		//Actualitzaci de dependncies entre preguntes
		String dependencies = p.get(id_pregunta).dependencies;
		if (dependencies != null && !dependencies.trim().equals("")){
			String[] depen = dependencies.split(",");
			if (depen[0].equals(codi_opcio) || depen[0].trim().equals("")){
				p.get(getIdPregunta(depen[1].trim())).bActiva = true;
			}
		}
		return codiOpcio;
	}
	
	/**
	 * Actualitza puntuaci espcies segons resposta 
	 * @param id_pregunta id de la pregunta
	 * @param codi_opcio codi de l'opcio
	 */
	public void actualitzaProbabilitats(Integer id_pregunta, String codi_opcio){
		Integer idOpcions = getIdOpcionsPregunta(p.get(id_pregunta).opcions);
		Integer pesPregunta = p.get(id_pregunta).pes;
		TotalPuntsSumats += pesPregunta;
		String keyDB = p.get(id_pregunta).keyDB;
		opcions mOpcions = o.get(idOpcions);
		mdbDadesOcellAdapter = new dbDadesOcellAdapter(mContext);
		mdbDadesOcellAdapter.open();
		
		//Si es una pregunta de caracterstica nica
		if (p.get(id_pregunta).tipus==pregunta.CARACTERISTICA){
			Integer factor = pesPregunta * mOpcions.relacions[0][0] /100;
			Cursor c = mdbDadesOcellAdapter.getDadesOcellsOnTop(1);
			c.moveToFirst();
			String idOcellOnTop = c.getString(c.getColumnIndex(dbDadesOcellAdapter.KEY_ID)).trim();
			if (codi_opcio.equals("si"))
				mdbDadesOcellAdapter.aplicaFactorUnic(factor, idOcellOnTop, "+");
			else
				mdbDadesOcellAdapter.aplicaFactorUnic(factor, idOcellOnTop,"-");
		} else if (p.get(id_pregunta).nom.equals("preguntaXorra")){ /*no actualitzem res*/}
		else {
			double factor;
			Integer idCodiOpcio = mOpcions.getIdOpcio(codi_opcio);
			for (int i = 0; i < mOpcions.codiOpcio.length; i++) {
				double relacio = mOpcions.cercaRelacio(idCodiOpcio, i);
				factor = relacio * pesPregunta /100;
				mdbDadesOcellAdapter.aplicaFactor(keyDB, Math.round(factor), mOpcions.codiOpcio[i]);
			}
		}
		mdbDadesOcellAdapter.close();
	}
	
	/**
	 * Calcula si es pot fer la pregunta de camp de prioritat 
	 * @return
	 */
	//
	public Integer preguntaPrioritat(){
		
		// Si no s'han fet suficients preguntes com per activar la pregunta ...
		if (preguntesFetes <= p.get(getIdPregunta("prioritat")).ordreMinim)
			return defs.NO;
		else {
			//Si entre els ocells per damunt de l'horitzo superior hi ha un que te camp pioritari, el preguntem
			Integer horitzoSuperior = cercarHoritzo(defs.PERCENT_HORITZO_SUPERIOR);
			mdbDadesOcellAdapter = new dbDadesOcellAdapter(mContext);
			mdbDadesOcellAdapter.open();
			Integer especiesSeleccionades = mdbDadesOcellAdapter.getNombreEspeciesHoritzo(horitzoSuperior);
			Cursor c = mdbDadesOcellAdapter.getDadesOcellsOnTop(especiesSeleccionades); 
			c.moveToFirst();
			Integer idPreguntaPrioritat = 0;
			boolean trobat = false; 
			while (!c.isAfterLast() && !trobat)
			{
				String cadenaPrioritat = c.getString(c.getColumnIndex(dbDadesOcellAdapter.KEY_PRIORITAT)).trim();
				if (cadenaPrioritat.length()!=0){
					String[] prioritats = cadenaPrioritat.split(",");
					for (int i = 0; i < prioritats.length && !trobat ; i++){
						Integer idPregunta = getIdPregunta(prioritats[i]);
						if (p.get(idPregunta).bActiva && !p.get(idPregunta).bFeta){
							trobat = true;
							idPreguntaPrioritat = idPregunta;
						}
					}
				}
				c.moveToNext();
			}
			c.close();
			mdbDadesOcellAdapter.close();
			if (trobat)
				return idPreguntaPrioritat;
			return defs.NO;
		}
	}
	
	/**
	 * Calcula si es pot fer la pregunta de la caracteristica nica
	 * En cas afirmatiu, modifica el text de la pregunta de caracterstica nica pel text de la pregunta per identificar l'espcie candidata 
	 * @return es pot fer?
	 */
	public boolean preguntaCaracteristicaUnica(){
		// Si no s'han fet suficients preguntes com per activar la pregunta ...
		
		if (preguntesFetes < p.get(getIdPregunta("caracteristicaUnica")).ordreMinim || p.get(getIdPregunta("caracteristicaUnica")).bActiva == false)
			return false;
		
		//Si el primer ocell t caracterstica nica i ja es permeten aquestes preguntes
		//Modifiquem el text de a pregunta pel que hi ha al camp de la BBDD
		mdbDadesOcellAdapter = new dbDadesOcellAdapter(mContext);
		mdbDadesOcellAdapter.open();
		Cursor c = mdbDadesOcellAdapter.getDadesOcellsOnTop(1); 
		c.moveToFirst();
		String pregunta = c.getString(c.getColumnIndex(dbDadesOcellAdapter.KEY_CARACTERISTICA_UNICA)).trim();
		mdbDadesOcellAdapter.close();
		if (!pregunta.equals("")){
			p.get(getIdPregunta("caracteristicaUnica")).text = pregunta; 
			return true;
		}
		return false;
	}
	
	
	/**
	 * Cerca la puntuaci que divideixi les espcies en l'horitzo demanat 
	 * @param tipus_horitzo tipus
	 * @return puntuaci
	 */
	public Integer cercarHoritzo(Integer tipus_horitzo){
		//TODO quina es la millor manera?
		//return TotalPuntsSumats/100*tipus_horitzo; 
		ArrayList<Integer> punts;
		mdbDadesOcellAdapter = new dbDadesOcellAdapter(mContext);
		mdbDadesOcellAdapter.open();
		punts = mdbDadesOcellAdapter.getPunts();
		Integer maxim = eines.maxim(punts);
		Integer minim = eines.minim(punts);
		Double horitzo = (double) (maxim - minim ) * tipus_horitzo / 100 ;
		horitzo = horitzo + minim;
		mdbDadesOcellAdapter.close();
		return horitzo.intValue();
	}
	
}
