package catalunya.de.ocells.db;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import android.app.ProgressDialog;
import android.content.ContentValues;
import android.content.Context;
import android.content.res.Resources;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.AsyncTask;
import android.text.TextUtils;
import android.util.Log;
import catalunya.de.ocells.R;
import catalunya.de.ocells.eines;
import catalunya.de.ocells.cerca.defs;

public abstract class dbAdapter { 
	private static final String TAG = "debugocells";
	private static Integer LINIA_PRIMER_OCELL = 1;
	private static Integer LINIA_ULTIM_OCELL = 596;

	private static final String DB_NOM = "OCELLS";
	private static final int DB_VERSIO = 529;
	
	private ProgressDialog pDialog;
	carregaAsincrona mCarregaAsincrona;
	
	/** OCELLS  Ordre dels camps del fitxer dels ocells *********************************/
	public static final Integer OCELLS_ROW_ID 			= 0;
	public static final Integer OCELLS_CODI_ICO 		= 1;
	public static final Integer OCELLS_NOM_CIENTIFIC 	= 2;
	public static final Integer OCELLS_NOM_CATALA 		= 3;
	public static final Integer OCELLS_NOM_ALTRES 		= 4;
	public static final Integer OCELLS_ADULT 			= 5;
	public static final Integer OCELLS_JOVE 			= 6;
	public static final Integer OCELLS_MASCLE 			= 7;
	public static final Integer OCELLS_FEMELLA 			= 8;
	public static final Integer OCELLS_NUPCIAL 			= 9;
	public static final Integer OCELLS_RESTA 			= 10;
	public static final Integer OCELLS_FAMILIA			= 11;
	public static final Integer OCELLS_TAMANY_MIN 		= 12;
	public static final Integer OCELLS_TAMANY_MAX 		= 13;
	public static final Integer OCELLS_TAMANY_MITJA 	= 14;
	public static final Integer OCELLS_TAMANY_CODI 		= 15;
	public static final Integer OCELLS_BEC_LLARGADA 	= 16;
	public static final Integer OCELLS_BEC_FORMA 		= 17;
	public static final Integer OCELLS_BEC_DOBLADA		= 18;
	public static final Integer OCELLS_BEC_COLOR		= 19;
	public static final Integer OCELLS_BEC_COLOR_PUNTA	= 20;
	public static final Integer OCELLS_POTES_PROPORCIO	= 21;
	public static final Integer OCELLS_POTES_COLOR		= 22;
	public static final Integer OCELLS_CUA_FORMA		= 23;
	public static final Integer OCELLS_FENOLOGIA_1		= 24;
	public static final Integer OCELLS_FENOLOGIA_2		= 25;
	public static final Integer OCELLS_FENOLOGIA_3		= 26;
	public static final Integer OCELLS_FENOLOGIA_4		= 27;
	public static final Integer OCELLS_FENOLOGIA_5		= 28;
	public static final Integer OCELLS_FENOLOGIA_6		= 29;
	public static final Integer OCELLS_FENOLOGIA_7		= 30;
	public static final Integer OCELLS_FENOLOGIA_8		= 31;
	public static final Integer OCELLS_FENOLOGIA_9		= 32;
	public static final Integer OCELLS_FENOLOGIA_10		= 33;
	public static final Integer OCELLS_FENOLOGIA_11		= 34;
	public static final Integer OCELLS_FENOLOGIA_12		= 35;
	public static final Integer OCELLS_ESTATUS			= 36;
	public static final Integer OCELLS_RAR				= 37;
	public static final Integer OCELLS_ASPECTE_CAP 		= 38;
	public static final Integer OCELLS_ASPECTE_SUPERIOR	= 39;
	public static final Integer OCELLS_ASPECTE_INFERIOR	= 40;
	public static final Integer OCELLS_ASPECTE_ALA		= 41;
	public static final Integer OCELLS_ASPECTE_CUA		= 42;
	public static final Integer OCELLS_COLOR_LINIA_ALA	= 43;
	public static final Integer OCELLS_COLOR_CRESTA		= 44;
	public static final Integer OCELLS_COLOR_FRONT		= 45;
	public static final Integer OCELLS_COLOR_PILI		= 46;
	public static final Integer OCELLS_COLOR_CELLA		= 47;
	public static final Integer OCELLS_COLOR_AURICULARS	= 48;
	public static final Integer OCELLS_COLOR_CLATELL	= 49;
	public static final Integer OCELLS_COLOR_DORS		= 50;
	public static final Integer OCELLS_COLOR_COBERTORES_MITJANES	= 51;
	public static final Integer OCELLS_COLOR_COBERTORES_GRANS		= 52;
	public static final Integer OCELLS_COLOR_PRIMARIES				= 53;
	public static final Integer OCELLS_COLOR_TERCIARIES				= 54;
	public static final Integer OCELLS_COLOR_CARPO					= 55;
	public static final Integer OCELLS_COLOR_SUPRACOBERTORES		= 56;
	public static final Integer OCELLS_COLOR_RECTRIUS_CENTRALS		= 57;
	public static final Integer OCELLS_COLOR_RECTRIUS_EXTERNES		= 58;
	public static final Integer OCELLS_COLOR_INFRACOBERTORES		= 59;
	public static final Integer OCELLS_COLOR_REGIO_ANAL				= 60;
	public static final Integer OCELLS_COLOR_REGIO_VENTRAL			= 61;
	public static final Integer OCELLS_COLOR_FLANCS		= 62;
	public static final Integer OCELLS_COLOR_PIT		= 63;
	public static final Integer OCELLS_COLOR_GOLA		= 64;
	public static final Integer OCELLS_COLOR_SUBBIGOTERA= 65;
	public static final Integer OCELLS_FORMA_COLL		= 66;
	public static final Integer OCELLS_HABITAT_AIGUAMOLLS	= 67;
	public static final Integer OCELLS_HABITAT_BOSC_RIBERA	= 68;
	public static final Integer OCELLS_HABITAT_SECA			= 69;
	public static final Integer OCELLS_HABITAT_COSTA		= 70;
	public static final Integer OCELLS_HABITAT_BOSC			= 71;
	public static final Integer OCELLS_HABITAT_PRATS_ALPINS	= 72;
	public static final Integer OCELLS_HABITAT_MAR			= 73;
	public static final Integer OCELLS_HABITAT_CULTIU_SECA	= 74;
	public static final Integer OCELLS_HABITAT_CULTIU_REGADIU= 75;
	public static final Integer OCELLS_HABITAT_RUPESTRE		= 76;
	public static final Integer OCELLS_PRIORITAT			= 77;
	public static final Integer OCELLS_CARACTERISTICA_UNICA	= 78;
	public static final Integer OCELLS_ACTIU				= 79;
	
		
	
	
	public static final String DB_CREACIO_TAULA_OCELLS =
			"create table " + dbOcellsAdapter.DB_TAULA_OCELLS + " ("
			+ dbOcellsAdapter.KEY_ID + " integer primary key autoincrement, "
			+ dbOcellsAdapter.KEY_CODI_ICO + " text, "
			+ dbOcellsAdapter.KEY_NOM_CIENTIFIC + " text, "
			+ dbOcellsAdapter.KEY_NOM_CATALA + " text,"
			+ dbOcellsAdapter.KEY_NOM_ALTRES + " text,"
			+ dbOcellsAdapter.KEY_NOM_NORMALITZAT + " text,"
			+ dbOcellsAdapter.KEY_FAMILIA + " text,"
			+ dbOcellsAdapter.KEY_ESTATUS + " text,"
			+ dbOcellsAdapter.KEY_RAR + " text,"
			+ dbOcellsAdapter.KEY_TAMANY_MIN + " numeric,"
			+ dbOcellsAdapter.KEY_TAMANY_MAX + " numeric);";
	
	public static final String DB_CREACIO_TAULA_LLISTES =
			"create table " + dbLlistesAdapter.DB_TAULA_LLISTES + " ("
			+ dbLlistesAdapter.KEY_ID + " integer primary key autoincrement, "
			+ dbLlistesAdapter.KEY_NOM + " text, "
			+ dbLlistesAdapter.KEY_DATA_HORA_CREACIO + " text, "
			+ dbLlistesAdapter.KEY_COORDENADES + " text);";

	public static final String DB_CREACIO_TAULA_OCELLS_LLISTA =
			"create table " + dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA + " ("
			+ dbOcellsLlistaAdapter.KEY_ID + " integer primary key autoincrement, "
			+ dbOcellsLlistaAdapter.KEY_ID_LLISTA + " numeric, "
			+ dbOcellsLlistaAdapter.KEY_ID_OCELL + " numeric, "
			+ dbOcellsLlistaAdapter.KEY_DATA_HORA_INSERCIO + " text, "
			+ dbOcellsLlistaAdapter.KEY_QUANTITAT + " numeric, "
			+ dbOcellsLlistaAdapter.KEY_ANOTACIO + " text);";
	
	public static final String DB_CREACIO_TAULA_DADES_OCELL =
			"create table " + dbDadesOcellAdapter.DB_TAULA_DADES_OCELL + " ("
			+ dbDadesOcellAdapter.KEY_ID + " integer primary key autoincrement, "
			+ dbDadesOcellAdapter.KEY_ID_OCELL + " text, "
			+ dbDadesOcellAdapter.KEY_ADULT + " text, "
			+ dbDadesOcellAdapter.KEY_JOVE + " text," 
			+ dbDadesOcellAdapter.KEY_MASCLE + " text,"
			+ dbDadesOcellAdapter.KEY_FEMELLA + " text,"
			+ dbDadesOcellAdapter.KEY_NUPCIAL + " text,"
			+ dbDadesOcellAdapter.KEY_RESTA + " text,"
			+ dbDadesOcellAdapter.KEY_TAMANY_CODI + " numeric,"
			+ dbDadesOcellAdapter.KEY_BEC_LLARGADA + " text,"
			+ dbDadesOcellAdapter.KEY_BEC_FORMA + " text,"
			+ dbDadesOcellAdapter.KEY_BEC_DOBLADA + " text,"
			+ dbDadesOcellAdapter.KEY_BEC_COLOR + " text,"
			+ dbDadesOcellAdapter.KEY_BEC_COLOR_PUNTA + " text,"
			+ dbDadesOcellAdapter.KEY_POTES_PROPORCIO + " text,"
			+ dbDadesOcellAdapter.KEY_POTES_COLOR + " text,"
			+ dbDadesOcellAdapter.KEY_CUA_FORMA + " text,"
			+ dbDadesOcellAdapter.KEY_FENOLOGIA_1 + " numeric,"
			+ dbDadesOcellAdapter.KEY_FENOLOGIA_2 + " numeric,"
			+ dbDadesOcellAdapter.KEY_FENOLOGIA_3 + " numeric,"
			+ dbDadesOcellAdapter.KEY_FENOLOGIA_4 + " numeric,"
			+ dbDadesOcellAdapter.KEY_FENOLOGIA_5 + " numeric,"
			+ dbDadesOcellAdapter.KEY_FENOLOGIA_6 + " numeric,"
			+ dbDadesOcellAdapter.KEY_FENOLOGIA_7 + " numeric,"
			+ dbDadesOcellAdapter.KEY_FENOLOGIA_8 + " numeric,"
			+ dbDadesOcellAdapter.KEY_FENOLOGIA_9 + " numeric,"
			+ dbDadesOcellAdapter.KEY_FENOLOGIA_10 + " numeric,"
			+ dbDadesOcellAdapter.KEY_FENOLOGIA_11 + " numeric,"
			+ dbDadesOcellAdapter.KEY_FENOLOGIA_12 + " numeric,"
			+ dbDadesOcellAdapter.KEY_RAR + " text,"
			+ dbDadesOcellAdapter.KEY_ASPECTE_CAP + " text,"
			+ dbDadesOcellAdapter.KEY_ASPECTE_SUPERIOR + " text,"
			+ dbDadesOcellAdapter.KEY_ASPECTE_INFERIOR + " text,"
			+ dbDadesOcellAdapter.KEY_ASPECTE_ALA + " text,"
			+ dbDadesOcellAdapter.KEY_ASPECTE_CUA + " text,"
			+ dbDadesOcellAdapter.KEY_TE_LINIA_ALA + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_LINIA_ALA + " text,"
			+ dbDadesOcellAdapter.KEY_TE_CRESTA + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_CRESTA + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_FRONT + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_PILI + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_CELLA + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_AURICULARS + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_CLATELL + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_DORS + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_COBERTORES_MITJANES + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_COBERTORES_GRANS + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_PRIMARIES + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_TERCIARIES + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_CARPO + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_SUPRACOBERTORES + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_RECTRIUS_CENTRALS + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_RECTRIUS_EXTERNES + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_INFRACOBERTORES + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_REGIO_ANAL + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_REGIO_VENTRAL + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_FLANCS + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_PIT + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_GOLA + " text,"
			+ dbDadesOcellAdapter.KEY_COLOR_SUBBIGOTERA + " text,"
			+ dbDadesOcellAdapter.KEY_FORMA_COLL + " text,"
			+ dbDadesOcellAdapter.KEY_HABITAT_AIGUAMOLLS + " numeric,"
			+ dbDadesOcellAdapter.KEY_HABITAT_BOSC_RIBERA + " numeric,"
			+ dbDadesOcellAdapter.KEY_HABITAT_SECA + " numeric,"
			+ dbDadesOcellAdapter.KEY_HABITAT_COSTA + " numeric,"
			+ dbDadesOcellAdapter.KEY_HABITAT_BOSC + " numeric,"
			+ dbDadesOcellAdapter.KEY_HABITAT_PRATS_ALPINS + " numeric,"
			+ dbDadesOcellAdapter.KEY_HABITAT_MAR + " numeric,"
			+ dbDadesOcellAdapter.KEY_HABITAT_CULTIU_SECA + " numeric,"
			+ dbDadesOcellAdapter.KEY_HABITAT_CULTIU_REGADIU + " numeric,"
			+ dbDadesOcellAdapter.KEY_HABITAT_RUPESTRE + " numeric,"
			+ dbDadesOcellAdapter.KEY_PRIORITAT + " text,"
			+ dbDadesOcellAdapter.KEY_CARACTERISTICA_UNICA + " text,"
			+ dbDadesOcellAdapter.KEY_PUNTS + " numeric);";
	
	private DBHelper mDBHelper;
	public SQLiteDatabase db;
	public Context context;

	public dbAdapter(Context context){
		this.context = context;
		this.mDBHelper = new DBHelper(this.context);		
	}

	public dbAdapter open() throws SQLException 
	{
	   db = mDBHelper.getWritableDatabase();
	   return this;
	}
	
	//TODO val la pena?
	public dbAdapter openLectura() throws SQLException 
	{
	   db = mDBHelper.getReadableDatabase();
	   return this;
	}

	public void close() 
	{
	    this.mDBHelper.close();
	}
	/**
	 * Carrega dades dels ocells
	 */
	public void carregarDadesOcells()  {
		pDialog = new ProgressDialog(context);
        pDialog.setProgressStyle(ProgressDialog. STYLE_HORIZONTAL);
        pDialog.setMessage(context.getResources().getString(R.string.db_adapter_dialeg_carrega_dades_titol));
        pDialog.setCancelable(false);
        pDialog.setMax(LINIA_ULTIM_OCELL - LINIA_PRIMER_OCELL);
        mCarregaAsincrona = new carregaAsincrona();
        mCarregaAsincrona.execute();
		
		
	}
	/**
	 * 
	 * @return Boole indica si la taula es plena o buida
	 */
	public boolean taulaOcellsPlena()  {
		
		Cursor cursor = db.rawQuery("SELECT count(*) FROM " + dbOcellsAdapter.DB_TAULA_OCELLS, null);
		if (cursor != null) {
		    cursor.moveToFirst();      
		    if (cursor.getInt(0) > 0) return true;
		}
		return false;
	}
	
	class DBHelper extends SQLiteOpenHelper {

		public DBHelper(Context context) {
			super(context, DB_NOM, null, DB_VERSIO);
		}
	
		@Override
		public void onCreate(SQLiteDatabase db) {
			Log.d(TAG, "Creant taules noves" );
			db.execSQL(DB_CREACIO_TAULA_LLISTES);
			db.execSQL(DB_CREACIO_TAULA_OCELLS_LLISTA);
			db.execSQL(DB_CREACIO_TAULA_OCELLS);
			db.execSQL(DB_CREACIO_TAULA_DADES_OCELL);
			//carregarDadesOcells(db);
			carregarLlistesExemple(db);
			carregarOcellsLlistaExemple(db);
			
		}
		
	
		private void carregarLlistesExemple(SQLiteDatabase db)  {
			ContentValues initialValues = new ContentValues();
			initialValues.put(dbLlistesAdapter.KEY_NOM, "Llista de mostra");
			initialValues.put(dbLlistesAdapter.KEY_DATA_HORA_CREACIO, "2013-04-25 12:34:33");
			initialValues.put(dbLlistesAdapter.KEY_COORDENADES, "31T3455,45874");
			db.insert(dbLlistesAdapter.DB_TAULA_LLISTES, null, initialValues);
		 }
		private void carregarOcellsLlistaExemple(SQLiteDatabase db)  {
			ContentValues initialValues = new ContentValues();
			initialValues.put(dbOcellsLlistaAdapter.KEY_ID_LLISTA, "1");
			initialValues.put(dbOcellsLlistaAdapter.KEY_ID_OCELL, "50");
			initialValues.put(dbOcellsLlistaAdapter.KEY_QUANTITAT, "2");
			db.insert(dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA, null, initialValues);
			
			ContentValues initialValues2 = new ContentValues();
			initialValues2.put(dbOcellsLlistaAdapter.KEY_ID_LLISTA, "1");
			initialValues2.put(dbOcellsLlistaAdapter.KEY_ID_OCELL, "51");
			initialValues2.put(dbOcellsLlistaAdapter.KEY_QUANTITAT, "1");
			db.insert(dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA, null, initialValues2);
			
			ContentValues initialValues3 = new ContentValues();
			initialValues3.put(dbOcellsLlistaAdapter.KEY_ID_LLISTA, "1");
			initialValues3.put(dbOcellsLlistaAdapter.KEY_ID_OCELL, "58");
			initialValues3.put(dbOcellsLlistaAdapter.KEY_QUANTITAT, "3");
			db.insert(dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA, null, initialValues3);
			
			ContentValues initialValues4 = new ContentValues();
			initialValues4.put(dbOcellsLlistaAdapter.KEY_ID_LLISTA, "1");
			initialValues4.put(dbOcellsLlistaAdapter.KEY_ID_OCELL, "61");
			initialValues4.put(dbOcellsLlistaAdapter.KEY_QUANTITAT, "5");
			initialValues4.put(dbOcellsLlistaAdapter.KEY_ANOTACIO, "Dos adults alimentant a tres cries, avistats a la vora del riu corb a l'altura de Golms");
			db.insert(dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA, null, initialValues4);
			
			ContentValues initialValues5 = new ContentValues();
			initialValues5.put(dbOcellsLlistaAdapter.KEY_ID_LLISTA, "1");
			initialValues5.put(dbOcellsLlistaAdapter.KEY_ID_OCELL, "63");
			initialValues5.put(dbOcellsLlistaAdapter.KEY_QUANTITAT, "1");
			db.insert(dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA, null, initialValues5);
			
			ContentValues initialValues6 = new ContentValues();
			initialValues6.put(dbOcellsLlistaAdapter.KEY_ID_LLISTA, "1");
			initialValues6.put(dbOcellsLlistaAdapter.KEY_ID_OCELL, "87");
			initialValues6.put(dbOcellsLlistaAdapter.KEY_QUANTITAT, "1");
			db.insert(dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA, null, initialValues6);
			
			ContentValues initialValues7 = new ContentValues();
			initialValues7.put(dbOcellsLlistaAdapter.KEY_ID_LLISTA, "1");
			initialValues7.put(dbOcellsLlistaAdapter.KEY_ID_OCELL, "79");
			initialValues7.put(dbOcellsLlistaAdapter.KEY_QUANTITAT, "1");
			db.insert(dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA, null, initialValues7);
			
			
		 }
		
		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
			Log.d(TAG, "Borrant taules antigues" );
			//db.execSQL("DROP TABLE IF EXISTS " + dbLlistesAdapter.DB_TAULA_LLISTES);
			//db.execSQL("DROP TABLE IF EXISTS " + dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA);
			db.execSQL("DROP TABLE IF EXISTS " + dbOcellsAdapter.DB_TAULA_OCELLS);
			db.execSQL("DROP TABLE IF EXISTS " + dbDadesOcellAdapter.DB_TAULA_DADES_OCELL);
			db.execSQL(DB_CREACIO_TAULA_OCELLS);
			db.execSQL(DB_CREACIO_TAULA_DADES_OCELL);

			//onCreate(db);
		
		}
		
	}
	
	 private class carregaAsincrona extends AsyncTask<Void, Integer, Boolean> {

		 @Override
         protected Boolean doInBackground(Void... params) {

        	 final Resources resources = context.getResources();
	         InputStream inputStream = resources.openRawResource(R.raw.ocells);
	         BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));
	         Log.d(TAG, "Iniciant carrega de fitxer");
             String line;
             int i=0;
             boolean bLiniesPerLlegir = true;
             while (bLiniesPerLlegir && i++ <= LINIA_ULTIM_OCELL){
             	try { line = reader.readLine();}
             	catch (IOException e){
	            	 	throw new RuntimeException(e);}
             	if (line == null) bLiniesPerLlegir = false;
             	else{
             		if (i < LINIA_PRIMER_OCELL ) continue;
             		String[] strings = TextUtils.split(line, ";");
             		tractarLinia(strings);
             		
             	}
             	if(isCancelled())
                     break;
             	 
             	publishProgress(i);
             }
             return true;
         }      
         
         @Override
         protected void onPreExecute() {
        	 pDialog.setProgress(0);
             pDialog.show();
         }
         
         @Override
         protected void onPostExecute(Boolean result) {
        	 if(result)
             {
                // if (pDialog.isShowing()) pDialog.dismiss();
        		 pDialog.dismiss();
                 //Toast.makeText(context, R.string.db_adapter_dialeg_carrega_dades_toast , Toast.LENGTH_SHORT).show();
             }
        	 
         }

         @Override
         protected void onProgressUpdate(Integer... values) {
        	 int progres = values[0].intValue();
        	 pDialog.setProgress(progres - LINIA_PRIMER_OCELL);
         }
         
         private void tractarLinia(String[] s){
        
 			long id_ocell;
 			String[] columns = new String[]{dbOcellsAdapter.KEY_ID};
 			String[] args = new String[] {s[OCELLS_NOM_CIENTIFIC].trim()};
 			Cursor cursor = db.query(dbOcellsAdapter.DB_TAULA_OCELLS, columns, dbOcellsAdapter.KEY_NOM_CIENTIFIC + "=?" ,args , null, null, null);
 			if (cursor.getCount()==0) {	//L'ocell encara no existeix
 				ContentValues initialValues = new ContentValues();
 				initialValues.put(dbOcellsAdapter.KEY_CODI_ICO, s[OCELLS_CODI_ICO].trim());
 				initialValues.put(dbOcellsAdapter.KEY_NOM_CIENTIFIC, s[OCELLS_NOM_CIENTIFIC].trim());
 				initialValues.put(dbOcellsAdapter.KEY_NOM_CATALA, s[OCELLS_NOM_CATALA].trim());
 				initialValues.put(dbOcellsAdapter.KEY_NOM_ALTRES, s[OCELLS_NOM_ALTRES].trim());
 				String nomsPerNormalitzar =  s[OCELLS_NOM_CATALA];
 				nomsPerNormalitzar += " " + s[OCELLS_NOM_CIENTIFIC];
 				nomsPerNormalitzar += " " + s[OCELLS_NOM_ALTRES];
 				nomsPerNormalitzar += " " + s[OCELLS_FAMILIA];
 				initialValues.put(dbOcellsAdapter.KEY_NOM_NORMALITZAT, eines.normalitza(nomsPerNormalitzar));
 				initialValues.put(dbOcellsAdapter.KEY_FAMILIA, s[OCELLS_FAMILIA].trim()); 
 				initialValues.put(dbOcellsAdapter.KEY_ESTATUS, s[OCELLS_ESTATUS].trim());
 				initialValues.put(dbOcellsAdapter.KEY_RAR, s[OCELLS_RAR].trim());
	 			initialValues.put(dbOcellsAdapter.KEY_TAMANY_MIN, Double.valueOf(s[OCELLS_TAMANY_MIN].trim().replace(",", ".")));
	 			initialValues.put(dbOcellsAdapter.KEY_TAMANY_MAX, Double.valueOf(s[OCELLS_TAMANY_MAX].trim().replace(",", ".")));
	 			
 	 	        id_ocell = db.insert(dbOcellsAdapter.DB_TAULA_OCELLS, null, initialValues );
 				
 				
 			} else { //L'ocell ja existeix. Nomes introduirem dades
 				cursor.moveToFirst();
 				id_ocell = Integer.valueOf(cursor.getString(0));
 			}
 			cursor.close();
 			
 			//Inserim les dades dels ocells actius
			if (s[OCELLS_ACTIU].equals("1")){ 
	 			ContentValues initialValues2 = new ContentValues();
	 			initialValues2.put(dbDadesOcellAdapter.KEY_ID_OCELL, id_ocell);
	 			initialValues2.put(dbDadesOcellAdapter.KEY_ADULT, s[OCELLS_ADULT].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_JOVE, s[OCELLS_JOVE].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_MASCLE, s[OCELLS_MASCLE].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FEMELLA, s[OCELLS_FEMELLA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_NUPCIAL, s[OCELLS_NUPCIAL].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_RESTA, s[OCELLS_RESTA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_TAMANY_CODI, s[OCELLS_TAMANY_CODI].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_BEC_LLARGADA, s[OCELLS_BEC_LLARGADA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_BEC_FORMA, s[OCELLS_BEC_FORMA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_BEC_DOBLADA, s[OCELLS_BEC_DOBLADA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_BEC_COLOR, s[OCELLS_BEC_COLOR].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_BEC_COLOR_PUNTA, s[OCELLS_BEC_COLOR_PUNTA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_POTES_PROPORCIO, s[OCELLS_POTES_PROPORCIO].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_CUA_FORMA, s[OCELLS_CUA_FORMA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_POTES_COLOR, s[OCELLS_POTES_COLOR].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FENOLOGIA_1, s[OCELLS_FENOLOGIA_1].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FENOLOGIA_2, s[OCELLS_FENOLOGIA_2].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FENOLOGIA_3, s[OCELLS_FENOLOGIA_3].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FENOLOGIA_4, s[OCELLS_FENOLOGIA_4].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FENOLOGIA_5, s[OCELLS_FENOLOGIA_5].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FENOLOGIA_6, s[OCELLS_FENOLOGIA_6].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FENOLOGIA_7, s[OCELLS_FENOLOGIA_7].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FENOLOGIA_8, s[OCELLS_FENOLOGIA_8].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FENOLOGIA_9, s[OCELLS_FENOLOGIA_9].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FENOLOGIA_10, s[OCELLS_FENOLOGIA_10].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FENOLOGIA_11, s[OCELLS_FENOLOGIA_11].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FENOLOGIA_12, s[OCELLS_FENOLOGIA_12].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_RAR, s[OCELLS_RAR].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_ASPECTE_CAP, s[OCELLS_ASPECTE_CAP].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_ASPECTE_SUPERIOR, s[OCELLS_ASPECTE_SUPERIOR].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_ASPECTE_INFERIOR, s[OCELLS_ASPECTE_INFERIOR].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_ASPECTE_ALA, s[OCELLS_ASPECTE_ALA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_ASPECTE_CUA, s[OCELLS_ASPECTE_CUA].trim());
	 			if (!s[OCELLS_COLOR_LINIA_ALA].trim().equals("")){
	 				 initialValues2.put(dbDadesOcellAdapter.KEY_TE_LINIA_ALA, "s");
	 			}
	 			else initialValues2.put(dbDadesOcellAdapter.KEY_TE_LINIA_ALA, "n");
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_LINIA_ALA, s[OCELLS_COLOR_LINIA_ALA].trim());
	 			 
	 			if (!s[OCELLS_COLOR_CRESTA].trim().equals("")){
	 				 initialValues2.put(dbDadesOcellAdapter.KEY_TE_CRESTA, "s");
	 			}
	 			else initialValues2.put(dbDadesOcellAdapter.KEY_TE_CRESTA, "n");
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_CRESTA, s[OCELLS_COLOR_CRESTA].trim());
	 			
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_FRONT, s[OCELLS_COLOR_FRONT].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_PILI, s[OCELLS_COLOR_PILI].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_CELLA, s[OCELLS_COLOR_CELLA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_AURICULARS, s[OCELLS_COLOR_AURICULARS].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_CLATELL, s[OCELLS_COLOR_CLATELL].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_DORS, s[OCELLS_COLOR_DORS].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_COBERTORES_MITJANES, s[OCELLS_COLOR_COBERTORES_MITJANES].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_COBERTORES_GRANS, s[OCELLS_COLOR_COBERTORES_GRANS].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_PRIMARIES, s[OCELLS_COLOR_PRIMARIES].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_TERCIARIES, s[OCELLS_COLOR_TERCIARIES].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_CARPO, s[OCELLS_COLOR_CARPO].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_SUPRACOBERTORES, s[OCELLS_COLOR_SUPRACOBERTORES].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_RECTRIUS_CENTRALS, s[OCELLS_COLOR_RECTRIUS_CENTRALS].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_RECTRIUS_EXTERNES, s[OCELLS_COLOR_RECTRIUS_EXTERNES].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_INFRACOBERTORES, s[OCELLS_COLOR_INFRACOBERTORES].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_REGIO_ANAL, s[OCELLS_COLOR_REGIO_ANAL].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_REGIO_VENTRAL, s[OCELLS_COLOR_REGIO_VENTRAL].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_FLANCS, s[OCELLS_COLOR_FLANCS].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_PIT, s[OCELLS_COLOR_PIT].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_GOLA, s[OCELLS_COLOR_GOLA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_COLOR_SUBBIGOTERA, s[OCELLS_COLOR_SUBBIGOTERA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_FORMA_COLL, s[OCELLS_FORMA_COLL].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_HABITAT_AIGUAMOLLS, s[OCELLS_HABITAT_AIGUAMOLLS].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_HABITAT_BOSC_RIBERA, s[OCELLS_HABITAT_BOSC_RIBERA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_HABITAT_SECA, s[OCELLS_HABITAT_SECA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_HABITAT_COSTA, s[OCELLS_HABITAT_COSTA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_HABITAT_BOSC, s[OCELLS_HABITAT_BOSC].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_HABITAT_PRATS_ALPINS, s[OCELLS_HABITAT_PRATS_ALPINS].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_HABITAT_MAR, s[OCELLS_HABITAT_MAR].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_HABITAT_CULTIU_SECA, s[OCELLS_HABITAT_CULTIU_SECA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_HABITAT_CULTIU_REGADIU, s[OCELLS_HABITAT_CULTIU_REGADIU].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_HABITAT_RUPESTRE, s[OCELLS_HABITAT_RUPESTRE].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_PRIORITAT, s[OCELLS_PRIORITAT].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_CARACTERISTICA_UNICA, s[OCELLS_CARACTERISTICA_UNICA].trim());
	 			initialValues2.put(dbDadesOcellAdapter.KEY_PUNTS, defs.PUNTS_INICIALS);
	 			db.insert(dbDadesOcellAdapter.DB_TAULA_DADES_OCELL, null, initialValues2 );
			}
 			
 		}
 	    
 /*        private long inserirOcell(ContentValues initialValues) {
 	        long id_ocell = db.insert(dbOcellsAdapter.DB_TAULA_OCELLS, null, initialValues );
 	        //Log.d(TAG, "Ocell afegit: " + id_ocell);
 	        if (id_ocell== -1){
 	        	Log.e(TAG, "Error afegint ocell" );
 			}
 	        return id_ocell;
 	    }
 		
         private long inserirDadesOcell(ContentValues initialValues) {
         //private long inserirDadesOcell(ContentValues initialValues, SQLiteDatabase db) {
 	        long id_dades_ocell = db.insert(dbDadesOcellAdapter.DB_TAULA_DADES_OCELL, null, initialValues );
 	        //Log.d(TAG, "Afegint dades ocell amb id: " + id_dades_ocell);
 	        if (id_dades_ocell== -1){
 	        	  Log.e(TAG, "Error afegint dades ocell" );
 			}
 	        return id_dades_ocell;
 	    }*/
	
	 }
}
