package catalunya.de.ocells.db;

import java.util.ArrayList;

import android.content.Context;
import android.database.Cursor;
import catalunya.de.ocells.cerca.defs;

/**
 * Classe per tractar les dades d'ocells contra la BBDD.
 * @author jsirera
 *
 */
public class dbDadesOcellAdapter extends dbAdapter{
	
	public static final String DB_TAULA_DADES_OCELL = "DADES_OCELL";
	
	/*** Noms dels camps de la BBDDD **************************************************/
	public static final String KEY_ID						= "_id"; 
	public static final String KEY_ID_OCELL 				= "idOcell";
	public static final String KEY_ADULT 					= "adult";
	public static final String KEY_JOVE 					= "jove";
	public static final String KEY_MASCLE 					= "mascle";
	public static final String KEY_FEMELLA 					= "femella";
	public static final String KEY_NUPCIAL					= "nupcial";
	public static final String KEY_RESTA 					= "resta";
	public static final String KEY_TAMANY_CODI 				= "tamanyCodi";
	public static final String KEY_BEC_LLARGADA 			= "becLlargada";
	public static final String KEY_BEC_FORMA 				= "becForma";
	public static final String KEY_BEC_DOBLADA 				= "becDoblada";
	public static final String KEY_BEC_COLOR 				= "becColor";
	public static final String KEY_BEC_COLOR_PUNTA 			= "becColorPunta";
	public static final String KEY_POTES_PROPORCIO 			= "potesProporcio";
	public static final String KEY_POTES_COLOR 				= "potesColor";
	public static final String KEY_CUA_FORMA				= "cuaForma";
	public static final String KEY_FENOLOGIA_1 				= "fenologia1";
	public static final String KEY_FENOLOGIA_2 				= "fenologia2";
	public static final String KEY_FENOLOGIA_3 				= "fenologia3";
	public static final String KEY_FENOLOGIA_4 				= "fenologia4";
	public static final String KEY_FENOLOGIA_5 				= "fenologia5";
	public static final String KEY_FENOLOGIA_6 				= "fenologia6";
	public static final String KEY_FENOLOGIA_7 				= "fenologia7";
	public static final String KEY_FENOLOGIA_8				= "fenologia8";
	public static final String KEY_FENOLOGIA_9 				= "fenologia9";
	public static final String KEY_FENOLOGIA_10 			= "fenologia10";
	public static final String KEY_FENOLOGIA_11 			= "fenologia11";
	public static final String KEY_FENOLOGIA_12 			= "fenologia12";
	public static final String KEY_RAR 						= "rar";
	public static final String KEY_ASPECTE_CAP				= "aspecteCap";
	public static final String KEY_ASPECTE_SUPERIOR			= "aspecteSuperior";
	public static final String KEY_ASPECTE_INFERIOR			= "aspecteInferior";
	public static final String KEY_ASPECTE_ALA				= "aspecteAla";
	public static final String KEY_ASPECTE_CUA				= "aspecteCua";
	public static final String KEY_TE_LINIA_ALA				= "teLiniaAla";
	public static final String KEY_COLOR_LINIA_ALA			= "colorLiniaAla";
	public static final String KEY_TE_CRESTA				= "teCresta";
	public static final String KEY_COLOR_CRESTA				= "colorCresta";
	public static final String KEY_COLOR_FRONT				= "colorFront";
	public static final String KEY_COLOR_PILI				= "colorPili";
	public static final String KEY_COLOR_CELLA				= "colorCella";
	public static final String KEY_COLOR_AURICULARS			= "colorAuriculars";
	public static final String KEY_COLOR_CLATELL			= "colorClatell";
	public static final String KEY_COLOR_DORS				= "colorDors";
	public static final String KEY_COLOR_COBERTORES_MITJANES= "colorCobertoresMitjanes";
	public static final String KEY_COLOR_COBERTORES_GRANS	= "colorCobertoresGrans";
	public static final String KEY_COLOR_PRIMARIES			= "colorPrimaries";
	public static final String KEY_COLOR_TERCIARIES			= "colorTerciaries";
	public static final String KEY_COLOR_CARPO				= "colorCarpo";
	public static final String KEY_COLOR_SUPRACOBERTORES	= "colorSupracobertores";
	public static final String KEY_COLOR_RECTRIUS_CENTRALS	= "colorRectriusCentrals";
	public static final String KEY_COLOR_RECTRIUS_EXTERNES	= "colorRectriusExternes";
	public static final String KEY_COLOR_INFRACOBERTORES	= "colorInfracobertores";
	public static final String KEY_COLOR_REGIO_ANAL			= "colorRegioAnal";
	public static final String KEY_COLOR_REGIO_VENTRAL		= "colorRegioVentral";
	public static final String KEY_COLOR_FLANCS				= "colorFlancs";
	public static final String KEY_COLOR_PIT				= "colorPit";
	public static final String KEY_COLOR_GOLA				= "colorGola";
	public static final String KEY_COLOR_SUBBIGOTERA		= "colorSubbigotera";
	public static final String KEY_FORMA_COLL				= "formaColl";
	public static final String KEY_HABITAT_AIGUAMOLLS		= "habitatAiguamolls";
	public static final String KEY_HABITAT_BOSC_RIBERA		= "habitatBoscRibera";
	public static final String KEY_HABITAT_SECA				= "habitatSeca";
	public static final String KEY_HABITAT_COSTA			= "habitatCosta";
	public static final String KEY_HABITAT_BOSC				= "habitatBosc";
	public static final String KEY_HABITAT_PRATS_ALPINS		= "habitatPratsAlpins";
	public static final String KEY_HABITAT_MAR				= "habitatMar";
	public static final String KEY_HABITAT_CULTIU_SECA		= "habitatCultiuSeca";
	public static final String KEY_HABITAT_CULTIU_REGADIU	= "habitatCultiuRegadiu";
	public static final String KEY_HABITAT_RUPESTRE			= "habitatRupestre";
	public static final String KEY_PRIORITAT				= "prioritat";
	public static final String KEY_CARACTERISTICA_UNICA		= "caracteristicaUnica";
	public static final String KEY_PUNTS					= "punts";
	//public static final String KEY_ACTIU					= "Actiu";
	
	public dbDadesOcellAdapter(Context c){
		super(c);
	}
	
	/**
	 * Retorna la quantitat d'especies per cada una de les diferents opcions disponibles d'una pregunta per damunt d'una puntuaci
	 * @param key_db camp de la base de dades a consultar
	 * @param horitzo punts mnims per contar l'espcie
	 * @return llista amb el nombre d'espcies per cada opcio
	 */
	//
	public ArrayList<Integer> getEspeciesPerOpcio(String key_db, Integer horitzo){
		String sql = "SELECT count(*) " +
				" FROM " + DB_TAULA_DADES_OCELL + 
				" WHERE " + KEY_PUNTS + " >= " + horitzo + 
				" GROUP BY " + key_db; 
		Cursor cursor = db.rawQuery(sql, null);
		ArrayList<Integer> llista = new ArrayList<Integer>();
		cursor.moveToFirst();
		while (!cursor.isAfterLast()) 
		{
			llista.add(cursor.getInt(0));
			cursor.moveToNext();
		}
		cursor.close();
		return llista;
	}
	
	/**
	 * Retorna les opcions disponibles per a una pregunta de les especies per damunt d'un horitzo 
	 * @param key_db camp de la base de dades a consultar
	 * @param horitzo punts minims per tindre en compte l'opcio
	 * @return  llista amb les opcions
	 */
	public ArrayList<String> getOpcionsUsades(String key_db, Integer horitzo){
		String sql = "SELECT " + key_db +
				" FROM " + DB_TAULA_DADES_OCELL + 
				" WHERE " + KEY_PUNTS + " >= " + horitzo +
				" GROUP BY " + key_db; 
		Cursor cursor = db.rawQuery(sql, null);
		ArrayList<String> llista = new ArrayList<String>();
		cursor.moveToFirst();
		while (!cursor.isAfterLast()) 
		{
			llista.add(cursor.getString(0));
			cursor.moveToNext();
		}
		cursor.close();
		return llista;
	}
	
	/**
	 * Aplica el factor calculat per a totes les especies que compleixen amb una caracteristica
	 * @param key_db camp de la taula per consultar
	 * @param factor factor per aplicar
	 * @param codi_opcio opci per puntuar
	 */
	public void aplicaFactor(String key_db, long factor, String codi_opcio){
		String sql = "UPDATE " + DB_TAULA_DADES_OCELL +
				" SET " + KEY_PUNTS + " = " + KEY_PUNTS + " + " + factor +
				" WHERE " + key_db + " = ?"; 
		db.execSQL(sql,  new Object[] {codi_opcio});
	}
	
	/**
	 * Aplica el factor (sumar o restar) per a l'especie indicada
	 * @param factor puntucaci per aplicar
	 * @param id_ocell ocell a aplicar el factor
	 * @param operacio operacio {+|-} 
	 */ 
	public void aplicaFactorUnic(Integer factor, String id_ocell, String operacio){
		String sql = "UPDATE " + DB_TAULA_DADES_OCELL +
				" SET " + KEY_PUNTS + " = " + KEY_PUNTS + operacio + factor +
				" WHERE " + KEY_ID + " = ?"; 
		db.execSQL(sql, new Object[] {id_ocell});

	}
	
	/**
	 * Baixa l'espcie de primera posici a segona 
	 */
	public void orAPlata(){
		String sql = "SELECT " + KEY_ID + ", " + KEY_PUNTS +
				" FROM " + DB_TAULA_DADES_OCELL + 
				" ORDER BY " + KEY_PUNTS + " DESC" +
				" LIMIT 2 "; 
		Cursor primerISegon = db.rawQuery(sql, null);
		primerISegon.moveToFirst();
		Integer idPrimer = primerISegon.getInt(primerISegon.getColumnIndex(KEY_ID));
		primerISegon.moveToNext();
		Integer puntsSegon = primerISegon.getInt(primerISegon.getColumnIndex(KEY_PUNTS));
		Integer novaPuntuacioPrimer = puntsSegon - 1;
		primerISegon.close();
		sql = "UPDATE " + DB_TAULA_DADES_OCELL +
				" SET " + KEY_PUNTS + " = " + novaPuntuacioPrimer +
				" WHERE " + KEY_ID + " = " + idPrimer ; 
		db.execSQL(sql);

	}
	
	
	/**
	 * Reseteja totes els punts 
	 */
	public void reiniciarPuntuacions(){
		String sql = "UPDATE " + DB_TAULA_DADES_OCELL +
				" SET " + KEY_PUNTS + " = ?"; 
		db.execSQL(sql,  new Object[] {defs.PUNTS_INICIALS});
	}
	
	/**
	 * Retorna les primeres dades trobades del ocell demanat
	 * @param id_ocell id de l'ocell
	 * @return cursor amb les dades de l'ocell
	 */
	public Cursor getDadesOcell(long id_ocell){
		String sql = "SELECT * " +
				" FROM " + DB_TAULA_DADES_OCELL + 
				" WHERE " + KEY_ID_OCELL + " = " + id_ocell;
		Cursor cursor = db.rawQuery(sql, null);
		return cursor;
	}
	
	/**
	 * Retorna 'n' ocells amb mes puntuacio
	 * @param nombre nombre d'ocells
	 * @return cursor amb les dades dels ocells
	 */
	public Cursor getDadesOcellsOnTop(Integer nombre){
		String sql = "SELECT * " +
				" FROM " + DB_TAULA_DADES_OCELL +
				" ORDER BY " + KEY_PUNTS + " DESC" +
				" LIMIT " + nombre; 
		Cursor cursor = db.rawQuery(sql, null);
		return cursor;
	}
	
	/**
	 * Retorna totes les puntuacions 
	 * @return llista amb totes les puntacions
	 */
	public ArrayList<Integer> getPunts(){
		String sql = "SELECT " + KEY_PUNTS + 
				" FROM " + DB_TAULA_DADES_OCELL;
		Cursor cursor = db.rawQuery(sql, null);
		ArrayList<Integer> llista = new ArrayList<Integer>();
		cursor.moveToFirst();
		while (!cursor.isAfterLast()) 
		{
			llista.add(cursor.getInt(0));
			cursor.moveToNext();
		}
		cursor.close();
		return llista;
	}
	
	/**
	 * Retorna el nombre despecies per damunt d'un horitzo 
	 * @param horitzo punts
	 * @return enter amb el nombre d'espcies
	 */
	public Integer getNombreEspeciesHoritzo(Integer horitzo){
		ArrayList<Integer> llista = getPunts();
		Integer nombre = 0;
		for (int i = 0; i < llista.size(); i++) {
			if (llista.get(i) >= horitzo)
				nombre++;
		}
		return nombre;
	}
	
	/**
	 * Retorna el nombre espcies iguals (sense tindre en compte sexe, edat...) que hi ha per damunt d'un horitz 
	 * @param horitzo punts
	 * @return enter amb nombre d'espcies iguals
	 */
	public Integer getNombreEspeciesIgualsHoritzo(Integer horitzo){
		String sql = "SELECT count(DISTINCT  " + KEY_ID_OCELL + ") " +
				" FROM " + DB_TAULA_DADES_OCELL +
				" WHERE " + KEY_PUNTS + " >= " + horitzo;
		Cursor cursor = db.rawQuery(sql, null);
		cursor.moveToFirst();
		return cursor.getInt(0);
		
	}
	
	
	
	/**
	 * Retorna totes les puntuacions amb el id i noms de l'ocell i tipus (mascle, adult...) ordenats per punts
	 * i amb limit (si es null --> sense limit) 
	 * @param l nombre d'espcies a retornar. Si es null --> no hi ha lmit
	 * @return dades dels ocells
	 */
	public Cursor getEstatCerca(Integer l){
		String limit = "";
		if (l!=null)
			limit = " LIMIT " + l;
		String sql = "SELECT o." + dbOcellsAdapter.KEY_ID + ", o." + dbOcellsAdapter.KEY_NOM_CATALA + ", o." + dbOcellsAdapter.KEY_NOM_CIENTIFIC + ", d." + KEY_PUNTS + 
				", d." + KEY_MASCLE + ", d." + KEY_FEMELLA + ", d." + KEY_ADULT + ", d." + KEY_JOVE + ", d." + KEY_NUPCIAL + ", d." + KEY_RESTA + 
				" FROM " + dbOcellsAdapter.DB_TAULA_OCELLS + " o, " + DB_TAULA_DADES_OCELL + " d " +
				" WHERE d." + KEY_ID_OCELL + "= o." + dbOcellsAdapter.KEY_ID +
				" ORDER BY d." + KEY_PUNTS + " DESC" + 	limit;
		Cursor cursor = db.rawQuery(sql, null);
		return cursor;
	}

}