package catalunya.de.ocells.db;

import java.text.SimpleDateFormat;
import java.util.Date;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import catalunya.de.ocells.eines;
/**
 * Classe per tractar les llistes contra la BBDD.
 * 
 * @author jsirera
 *
 */

public class dbLlistesAdapter extends dbAdapter{
	
	public static final String DB_TAULA_LLISTES = "LLISTES";
	
	/*** Noms dels camps de la BBDDD **************************************************/
	public static final String KEY_ID = "_id";
	public static final String KEY_NOM = "nom";
	public static final String KEY_DATA_HORA_CREACIO = "dataHoraCreacio";
	public static final String KEY_COORDENADES = "coordenades";
		
	public dbLlistesAdapter(Context c){
		super(c);
	}
	
	/**
	 * Crea una llista
	 * @param nom nom de la llista
	 * @param nova indica si la llista s nova o s una copia
	 * @return id de la nova llista
	 */
	public long crearLlista(String nom, boolean nova){
		SimpleDateFormat dateFormat = new SimpleDateFormat(eines.FORMAT_DATA_HORA_DB); 
		Date date = new Date();
		ContentValues initialValues = new ContentValues();
		initialValues.put(dbLlistesAdapter.KEY_NOM,  nom);
		initialValues.put(dbLlistesAdapter.KEY_DATA_HORA_CREACIO, dateFormat.format(date));
		initialValues.put(dbLlistesAdapter.KEY_COORDENADES, "31T3455,45874");
		long id = db.insert(DB_TAULA_LLISTES, null, initialValues);
		if (nova){
			ContentValues valors = new ContentValues();
			valors.put("nom",nom + " " + id);
			db.update(DB_TAULA_LLISTES, valors, KEY_ID + "=" + id, null);
		}
		return id;
	}
	
	/**
	 * Esborra la llista
	 * @param id_llista llista per esborrar
	 */
	public void esborrarLlista(long id_llista){
		//Borrem la llista i tots els ocells relacionats
		db.delete(DB_TAULA_LLISTES, KEY_ID + "=" + id_llista, null);
		db.delete(dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA, dbOcellsLlistaAdapter.KEY_ID_LLISTA + "=" + id_llista, null);
	}
	
	/**
	 * Extreu totes les llistes de la base de dades
	 * @param ordre ordre de les llistes
	 * @return cursor amb totes les llistes
	 */
	public Cursor llistaTot(int ordre){
		String columna_ordre;
		switch (ordre) {
			case(0): columna_ordre = KEY_NOM + " COLLATE NOCASE";
			break;
			case(1): columna_ordre = KEY_DATA_HORA_CREACIO;
			break;
			default: columna_ordre = KEY_DATA_HORA_CREACIO + " DESC";
		}
		String sql = "SELECT "+ DB_TAULA_LLISTES + "." + KEY_ID + ", " + DB_TAULA_LLISTES + "." + KEY_NOM + ", " + DB_TAULA_LLISTES + "." + KEY_DATA_HORA_CREACIO+ ", COUNT (" + dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA + "." + dbOcellsLlistaAdapter.KEY_ID + ") AS num_ocells " +
					" FROM " + DB_TAULA_LLISTES + 
					" LEFT OUTER JOIN " + dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA +
					" ON " + DB_TAULA_LLISTES + "." + KEY_ID + "=" + dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA + "." + dbOcellsLlistaAdapter.KEY_ID_LLISTA +
					" GROUP BY " + DB_TAULA_LLISTES + "." + KEY_ID +
					" ORDER BY " + columna_ordre ;
		Cursor cursor = db.rawQuery(sql, null);
		return cursor;
	}
	
	/**
	 * Retorna el nom d'una llista
	 * @param id_llista id de la llista consultada
	 * @return nom de la llista
	 */
	public String getNomLlista(long id_llista){
		String[] args = new String[] {String.valueOf(id_llista)};
		Cursor cursor = db.query(DB_TAULA_LLISTES,new String[] {KEY_NOM} ,KEY_ID+"=?",args , null, null, null);
		cursor.moveToFirst();
		return cursor.getString(0);
	}
	
	/**
	 * Modifica el nom d'una llista
	 * @param id_llista llista a modificar
	 * @param nom_nou nom nou per a la llista
	 */
	public void setNomLlista(long id_llista, String nom_nou){
		ContentValues camps = new ContentValues();
		camps.put(KEY_NOM, nom_nou);
		String[] args = new String[] {String.valueOf(id_llista)};
		db.update(DB_TAULA_LLISTES, camps, KEY_ID+"=?", args);
	}
	
	/**
	 * Retorna tots els camps d'una llista
	 * @param id_llista llista per consultar
	 * @return cursor amb la informaci de la llista
	 */
	public Cursor getLlista(long id_llista){
		String sql = "SELECT "+ DB_TAULA_LLISTES + "." + KEY_ID + ", " + DB_TAULA_LLISTES + "." + KEY_NOM + ", " + DB_TAULA_LLISTES + "." + KEY_DATA_HORA_CREACIO+ ", COUNT (" + dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA + "." + dbOcellsLlistaAdapter.KEY_ID + ") AS num_ocells " +
					" FROM " + DB_TAULA_LLISTES + 
					" LEFT OUTER JOIN " + dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA +
					" ON " + DB_TAULA_LLISTES + "." + KEY_ID + "=" + dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA + "." + dbOcellsLlistaAdapter.KEY_ID_LLISTA + 
					" WHERE " +  DB_TAULA_LLISTES + "." + KEY_ID + "=" + id_llista +
					" GROUP BY " + DB_TAULA_LLISTES + "." + KEY_ID ;
		Cursor cursor = db.rawQuery(sql, null);
		return cursor;
	}
	
	/**
	 * Copia una llista amb un text afegit al davant del nom
	 * @param id_llista llista per copiar
	 * @param afegit_al_nom text per afegir al davant del nom
	 * @return
	 */
	public long copiarLlista(Long id_llista, String afegit_al_nom){
		String nomLlista = getNomLlista(id_llista);
		Long idNovaLlista = crearLlista(afegit_al_nom + " " + nomLlista, false);
		
		SimpleDateFormat dateFormat = new SimpleDateFormat(eines.FORMAT_DATA_HORA_DB); 
		Date date = new Date();
		String dataHotaActual = dateFormat.format(date);
		
		String[] args = new String[] {String.valueOf(id_llista)};
		Cursor cursor = db.query(dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA,new String[] {dbOcellsLlistaAdapter.KEY_ID_OCELL} ,dbOcellsLlistaAdapter.KEY_ID_LLISTA+"=?",args , null, null, null);
		cursor.moveToFirst();
		
		while (!cursor.isAfterLast()) 
		{
			ContentValues initialValues = new ContentValues();
			initialValues.put(dbOcellsLlistaAdapter.KEY_ID_LLISTA, idNovaLlista);
			initialValues.put(dbOcellsLlistaAdapter.KEY_ID_OCELL, cursor.getString(cursor.getColumnIndex(dbOcellsLlistaAdapter.KEY_ID_OCELL)));
			initialValues.put(dbOcellsLlistaAdapter.KEY_DATA_HORA_INSERCIO,dataHotaActual);
			initialValues.put(dbOcellsLlistaAdapter.KEY_QUANTITAT, 0);
			db.insert(dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA, null, initialValues);
			cursor.moveToNext();
		}
		cursor.close();
		return idNovaLlista;
	}
	
}