package catalunya.de.ocells.db;

import android.content.Context;
import android.database.Cursor;
/**
 * Classe per tractar els ocells contra la BBDD.
 * @author jsirera
 *
 */

public class dbOcellsAdapter extends dbAdapter{
	
	public static final String DB_TAULA_OCELLS = "OCELLS";
	
	/*** Noms dels camps de la BBDDD **************************************************/
	public static final String KEY_ID = "_id";
	public static final String KEY_CODI_ICO = "codiIco";
	public static final String KEY_NOM_CIENTIFIC = "nomCientific";
	public static final String KEY_NOM_CATALA = "nomCatala";
	public static final String KEY_NOM_ALTRES = "nomAltres";
	public static final String KEY_NOM_NORMALITZAT = "nomNormalitzat";
	public static final String KEY_FAMILIA = "familia";
	public static final String KEY_ESTATUS = "estatus";
	public static final String KEY_RAR = "rar";
	public static final String KEY_TAMANY_MIN = "tamanyMin";
	public static final String KEY_TAMANY_MAX = "tamanyMax";
	
	public dbOcellsAdapter(Context c){
		super(c);
	}
	
	/**
	 * Retorna tots els ocells d'una llista
	 * @param id_llista llista per consultar
	 * @param ordre ordre dels ocells retornats
	 * @return cursor amb els ocells de la llista
	 */
	public Cursor getOcellsLlista(long id_llista, int ordre){ 
		String columna_ordre;
		switch (ordre) {
			case(0): columna_ordre = "o." + KEY_NOM_NORMALITZAT + " COLLATE NOCASE";
			break;
			case(1): columna_ordre = "o." + KEY_NOM_CIENTIFIC;
			break;
			case(2): columna_ordre = "o." + KEY_ID; //ordre per familia, tal com estan a la BBDD
			break;
			default: columna_ordre = "ol." + dbOcellsLlistaAdapter.KEY_DATA_HORA_INSERCIO;
		}
		String sql = "SELECT" + 
				" o."  + KEY_ID +
				", o."  + KEY_NOM_CATALA + 
				", o."  + KEY_NOM_CIENTIFIC +
				", ol." + dbOcellsLlistaAdapter.KEY_QUANTITAT +
				", ol." + dbOcellsLlistaAdapter.KEY_ANOTACIO +
				" FROM " + DB_TAULA_OCELLS + " AS o, " + dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA + " AS ol " +
				" WHERE" +
					" ol." + dbOcellsLlistaAdapter.KEY_ID_LLISTA + "=" + id_llista + 
					" AND o." + KEY_ID + "=ol." + dbOcellsLlistaAdapter.KEY_ID_OCELL +
				" ORDER BY " + columna_ordre;
		Cursor cursor = db.rawQuery(sql, null);
		return cursor;
	}
	
	
	/**
	 * Retorna ocells per a la cerca escrita en qualsevol camp de noms
	 * @param query cadena a cercar
	 * @return cursor amb ocells trobats
	 */
	public Cursor cercaPerNoms(String query){ 
		String[] columns = new String[]{KEY_ID,KEY_CODI_ICO, KEY_NOM_CIENTIFIC,KEY_NOM_CATALA, KEY_FAMILIA};
		String sql = query.replace("\"","'");
		//TODO Que retorni primer els ocells que comencen pe la paraula i despre stota la resta
		//Cursor cursor = db.query(DB_TAULA_OCELLS, columns, KEY_NOM_CATALA + " LIKE \"%" + sql + "%\" OR " + KEY_NOM_CIENTIFIC + " LIKE \"%" + sql + "%\" OR " + KEY_NOM_NORMALITZAT + " LIKE \"%" + sql + "%\"",null , KEY_NOM_CATALA, null, KEY_NOM_CATALA);
		Cursor cursor = db.query(DB_TAULA_OCELLS, columns, KEY_NOM_NORMALITZAT + " LIKE \"%" + sql + "%\"",null , KEY_NOM_CATALA, null, KEY_NOM_CATALA);
		return cursor;
	}

	/**
	 * Retorna ocell per cerca de veu
	 * @param query cadena a cercar
	 * @return cursor amb ocells trobats
	 */
	public Cursor cercaPerVeu(String query){ 
		String[] columns = new String[]{KEY_ID,KEY_CODI_ICO, KEY_NOM_CIENTIFIC,KEY_NOM_CATALA, KEY_FAMILIA};
		String sql = query.replace("\"","'");
		Cursor cursor = db.query(DB_TAULA_OCELLS, columns, KEY_NOM_CATALA + " = \"" + query + "\" OR " + KEY_NOM_CIENTIFIC + " = \"" + query + "\"",null , KEY_NOM_CATALA, null, KEY_NOM_CATALA);
		return cursor;
	}

	/**
	 * Retorna totes les especies
	 * @return cursor amb totes les especies
	 */
	public Cursor llistaTots(int ordre){
		String columna_ordre;
		switch (ordre) {
			case(0): columna_ordre = KEY_NOM_NORMALITZAT + " COLLATE NOCASE";
			break;
			case(1): columna_ordre = KEY_NOM_CIENTIFIC;
			break;
			case(2): columna_ordre = KEY_ID; //ordre per familia, tal com estan a la BBDD
			break;
			case(3): columna_ordre = KEY_TAMANY_MAX; //ordre per familia, tal com estan a la BBDD
			break;
			default: columna_ordre = KEY_ID;
		}
		String[] columns = new String[]{KEY_ID, KEY_NOM_CIENTIFIC,KEY_NOM_CATALA};
		Cursor cursor = db.query(DB_TAULA_OCELLS, columns,null, null, null, null, columna_ordre);
		return cursor;
	}
	
	/**
	 * Retorna totes les dades d'una espcie per a la fitxa
	 * @param id id de l'espcie
	 * @return cursor amb les dades
	 */
	public Cursor dadesOcell(long id){
		//String[] columns = new String[]{KEY_ID,KEY_CODI_ICO, KEY_NOM_CIENTIFIC,KEY_NOM_CATALA, KEY_FAMILIA, KEY_ESTATUS, KEY_TAMANY_MIN, KEY_TAMANY_MAX};
		String[] columns = new String[]{"*"};
		String[] args = new String[] {String.valueOf(id)};
		Cursor cursor = db.query(DB_TAULA_OCELLS, columns,KEY_ID+"=?",args , null, null, null);
		return cursor;
	}

}