package catalunya.de.ocells.db;

import java.text.SimpleDateFormat;
import java.util.Date;

import catalunya.de.ocells.eines;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;

/**
 * Classe per tractar els ocells de les llistes contra la BBDD.
 * @author jsirera
 *
 */
public class dbOcellsLlistaAdapter extends dbAdapter{
	
	public static final String DB_TAULA_OCELLS_LLISTA = "OCELLS_LLISTA";
	
	/*** Noms dels camps de la BBDDD **************************************************/
	public static final String KEY_ID = "_id";
	public static final String KEY_ID_LLISTA = "idLlista";
	public static final String KEY_ID_OCELL = "idOcell";
	public static final String KEY_DATA_HORA_INSERCIO = "dataHoraInsercio";
	public static final String KEY_QUANTITAT = "quantitat";
	public static final String KEY_ANOTACIO = "anotacio";
	
		
	public dbOcellsLlistaAdapter(Context c){
		super(c);
	}
	
	/**
	 * Afegeix una espcie a una llista
	 * @param id_llista llista on s'afegir l'espcie
	 * @param id_ocell especie per afegir
	 * @param quantitat nombre 
	 * @return quantitat nova
	 */
	public long afegirOcellLlista (long id_llista, long id_ocell, int quantitat){
		//Si ja existeix sumem la quantitat
		int nombreActual = getQuantitatEspecieEnLlista(id_llista, id_ocell);
		if (nombreActual==-1){
			SimpleDateFormat dateFormat = new SimpleDateFormat(eines.FORMAT_DATA_HORA_DB); 
			Date date = new Date();
			ContentValues initialValues = new ContentValues();
			initialValues.put(dbOcellsLlistaAdapter.KEY_ID_LLISTA, id_llista);
			initialValues.put(dbOcellsLlistaAdapter.KEY_ID_OCELL, id_ocell);
			initialValues.put(dbOcellsLlistaAdapter.KEY_DATA_HORA_INSERCIO, dateFormat.format(date));
			initialValues.put(dbOcellsLlistaAdapter.KEY_QUANTITAT, quantitat);
			return db.insert(dbOcellsLlistaAdapter.DB_TAULA_OCELLS_LLISTA, null, initialValues);
		} else {
			int nombreNou = nombreActual + quantitat;
			ContentValues camps = new ContentValues();
			camps.put(KEY_QUANTITAT, nombreNou);
			String[] args = new String[] {String.valueOf(id_llista),String.valueOf(id_ocell) };
			db.update(DB_TAULA_OCELLS_LLISTA, camps, KEY_ID_LLISTA+"=? AND "+ KEY_ID_OCELL +"=?", args);
			return nombreNou;
		}
	}
	
	/**
	 * Esborra l'espcie de la llista
	 * @param id_llista llista
	 * @param id_ocell espcie
	 */
	public void esborraOcellLlista(long id_llista, long id_ocell){
		String[] args = new String[] {String.valueOf(id_llista), String.valueOf(id_ocell)};
		db.delete(DB_TAULA_OCELLS_LLISTA, KEY_ID_LLISTA + "=? AND " + KEY_ID_OCELL + " = ?", args);
	}
	
	/**
	 * Retorna l'anotacio de l'ocell per a la llista 
	 * @param id_llista llista
	 * @param id_ocell ocell
	 * @return anotaci
	 */
	public String getAnotacioOcell(long id_llista, long id_ocell){
		String[] args = new String[] {String.valueOf(id_llista), String.valueOf(id_ocell)};
		Cursor cursor = db.query(DB_TAULA_OCELLS_LLISTA,new String[] {KEY_ANOTACIO} ,KEY_ID_LLISTA+"=? AND " + KEY_ID_OCELL+"=?",args , null, null, null);
		if (cursor.getCount()==0) return "";
		else {
			cursor.moveToFirst();
			return cursor.getString(0);
		}
	}
	
	/**
	 * Estableix anotacio de l'espcie per a la llista 
	 * @param id_llista llista
	 * @param id_ocell espcie
	 * @param anotacio text de l'anotaci
	 */
	public void setAnotacioOcell(long id_llista, long id_ocell, String anotacio){
		ContentValues camps = new ContentValues();
		camps.put(KEY_ANOTACIO, anotacio);
		String[] args = new String[] {String.valueOf(id_llista),String.valueOf(id_ocell) };
		db.update(DB_TAULA_OCELLS_LLISTA, camps, KEY_ID_LLISTA+"=? AND "+ KEY_ID_OCELL +"=?", args);
	}
	
	/**
	 * Estableix el nombre d'ndividus de la mateixa especie vistos en una llista 
	 * @param id_llista llista
	 * @param id_ocell espcie
	 * @param nombre quantitat
	 */
	public void setNombreOcellsLlista(long id_llista, long id_ocell, int nombre){
		ContentValues camps = new ContentValues();
		camps.put(KEY_QUANTITAT, nombre);
		String[] args = new String[] {String.valueOf(id_llista),String.valueOf(id_ocell) };
		db.update(DB_TAULA_OCELLS_LLISTA, camps, KEY_ID_LLISTA+"=? AND "+ KEY_ID_OCELL +"=?", args);
	}
	
	/**
	 * Retorna el nombre d'una espcie en una llista
	 * @param id_llista llista
	 * @param id_ocell espcie
	 * @return quantitat
	 */
	public int getQuantitatEspecieEnLlista(long id_llista, long id_ocell){
		String[] args = new String[] {String.valueOf(id_ocell),String.valueOf(id_llista)};
		Cursor cursor = db.query(DB_TAULA_OCELLS_LLISTA,new String[] {KEY_QUANTITAT} ,KEY_ID_OCELL+"=? AND " + KEY_ID_LLISTA+"=?",args , null, null, null);
		if (cursor.getCount()==0) return -1; //No existeix l'ocell a la llista
		else {
			cursor.moveToFirst();
			return Integer.valueOf(cursor.getString(0));
		}
	}
	

}