package catalunya.de.ocells;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import android.content.SharedPreferences;
import android.media.AudioManager;
import android.media.ToneGenerator;
import android.preference.PreferenceManager;
import android.text.format.DateUtils;
import android.util.Log;
import android.view.Display;
import android.view.WindowManager;
/**
 * Classe eines amb diferents mtodes esttics per resoltre algunes tasques
 * @author jsirera
 *
 */
public class eines {

	 public static final String FORMAT_DATA_HORA_DB = "yyyy-MM-dd HH:mm:ss";
	 public static final String FORMAT_DATA_HORA_DIES_ANTERIORS ="dd/MM/yyyy";
	 public static final String FORMAT_DATA_HORA_AVUI ="HH:mm";
	

	 /**
	  * Retorna numero mes per als calculs (agost = 8)
	  * @return mes numeric
	  */
	 public static String getMesNumeric(){
		 Date date = new Date();
		 String dateFormat = "M";
		 SimpleDateFormat mes = new SimpleDateFormat(dateFormat);
		 String sMes = mes.format(date);
		 return sMes;
	 }
	
	 
	 /**
	  * Retorna data/hora en format reduit a partir de format BBDD.
	  * Si la data es de dies anteriors, retorna en format dd/mm/aaaa, 
	  * si es d'avui retorna hora hh:mm 
	  * @param original data en format DB
	  * @return string amb la data o hora
	  */
	 public static String convertirDataHoraRelativa(String original){
		 SimpleDateFormat formatOrigen = new SimpleDateFormat(FORMAT_DATA_HORA_DB);
		 Date origen;
		 try {
			origen = formatOrigen.parse(original);
		 } catch (ParseException e) {
			return null;
		 }
		 
		 String format;
		 Date now = new Date();
 		 if (DateUtils.isToday(origen.getTime()))
			format = FORMAT_DATA_HORA_AVUI;
		 else
			format = FORMAT_DATA_HORA_DIES_ANTERIORS;
		 
		 SimpleDateFormat formatDesti = new SimpleDateFormat(format);
		 return formatDesti.format(origen);
		 
		 
	 }
	 
	 /**
	  * Retorna data/hora en format reduit a partir de format BBDD.
	  * @param original original data en format DB
	  * @return string amb la data
	  */
	 public static String convertirDataHoraSimple(String original){
		 		
		SimpleDateFormat formatOrigen = new SimpleDateFormat(FORMAT_DATA_HORA_DB);
	    SimpleDateFormat formatDesti = new SimpleDateFormat(FORMAT_DATA_HORA_DIES_ANTERIORS);
		
    	Date origen;
		try {
			origen = formatOrigen.parse(original);
		} catch (ParseException e) {
			return null;
		}
		return formatDesti.format(origen);
	 }
	
	 /**
	  * Canvia carcter en blanc per gui baix.
	  * @param original text original
	  * @return text modificat
	  */
	 public static String nomFitxer(String original){
		 String nom_fitxer = original.replace(" ", "_").toLowerCase();
		 return nom_fitxer;
	 }
		
	 
	 
	 /**
	  * Funcio estadstica per al clcul de la desviaci estandard
	  * @param valors ArrayList amb nombres enters
	  * @return resultat
	  */
	 public static double desviacioEstandard(ArrayList<Integer> valors){
		 return Math.sqrt(variancia(valors));
	 }
	 
	 /**
	  * Funcio estadstica per al clcul de la variancia 
	  * @param valors ArrayList amb nombres enters
	  * @return resultat
	  */
	 public static double variancia(ArrayList<Integer> valors){
		 Double mitjana = mitjana(valors);
		 Double diferencia;
		 double suma = 0 ;
		 for (int i = 0; i < valors.size(); i++){
			 diferencia = Math.pow(valors.get(i) - mitjana, 2);
			 suma += diferencia;
		 }
		 return suma / valors.size();
	 }

	 /**
	  * Funcio estadstica per al clcul de la mitjana
	  * @param valors ArrayList amb nombres enters
	  * @return resultat
	  */
	 public static double mitjana(ArrayList<Integer> valors){
		 double mitjana;
		 Integer suma = 0 ;
		 for (int i = 0; i < valors.size(); i++){
			 suma += valors.get(i);
			 
		 }
		 mitjana = suma / valors.size();
		 return mitjana;
	 }
	 
	 /**
	  * Funcio estadstica per al clcul del mxim
	  * @param valors ArrayList amb nombres enters
	  * @return resultat
	  */
	 public static Integer maxim(ArrayList<Integer> valors){
		 Integer maxim = 0;
		 for (int i = 0; i < valors.size(); i++){
			 if (valors.get(i) > maxim)
				 maxim = valors.get(i) ;
		 }
		 return maxim;
	 }
	 
	 /**
	  * Funcio estadstica per al clcul del minim
	  * @param valors ArrayList amb nombres enters
	  * @return resultat
	  */
	 public static Integer minim(ArrayList<Integer> valors){
		 Integer minim = valors.get(0);
		 for (int i = 0; i < valors.size(); i++){
			 if (valors.get(i) < minim)
				 minim = valors.get(i) ;
		 }
		 return minim;
	 }
	 
	 /**
	 * Reproducci d'un beep
	 */
	 public static void reproduirSo(){
		 final ToneGenerator tg = new ToneGenerator(AudioManager.STREAM_NOTIFICATION, 100);
         tg.startTone(ToneGenerator.TONE_PROP_BEEP);
         
	 }
	
	  /**
	  * Obt l'amplada en pixels del dispositiu
	  * @param mWindowManager
	  * @return amplada
	  */
	 //Obte l'amplada del dipositiu
	 public static Integer getAmpladaDispositiu(WindowManager  mWindowManager){
		Display display = mWindowManager.getDefaultDisplay(); 
		return display.getWidth();		 
	 }
	 
	 /**
	  * Obt l'altura en pixels del dispositiu
	  * @param mWindowManager
	  * @return altura
	  */
	 public static Integer getAlturaDispositiu(WindowManager  mWindowManager){
		Display display = mWindowManager.getDefaultDisplay(); 
		return display.getHeight();		 

	 }	 
	 
	 /**
	  * Mtode per a la normalitzaci d'un text.
	  * Desant a la BBDD el text normalitzat poder realitzar consultes amb el text introduit per l'usuari.
	  * Al tanto, si ets filleg no continuis llegint...
	  * @param origen text original
	  * @return text normalitzat
	  */
	 public static String normalitza(String origen){
		 String nomNormalitzat = origen.toLowerCase();
		 nomNormalitzat = nomNormalitzat.replace("", "a");
		 nomNormalitzat = nomNormalitzat.replace("", "a");
		 nomNormalitzat = nomNormalitzat.replace("", "e");
		 nomNormalitzat = nomNormalitzat.replace("", "e");
		 nomNormalitzat = nomNormalitzat.replace("", "i");
		 nomNormalitzat = nomNormalitzat.replace("", "o");
		 nomNormalitzat = nomNormalitzat.replace("", "o");
		 nomNormalitzat = nomNormalitzat.replace("", "u");
		 nomNormalitzat = nomNormalitzat.replace("v", "b");
		 nomNormalitzat = nomNormalitzat.replace("y", "i");
		 nomNormalitzat = nomNormalitzat.replace("", "c");
		 nomNormalitzat = nomNormalitzat.replace("j", "g");
		 nomNormalitzat = nomNormalitzat.replace("l", "");
		 nomNormalitzat = nomNormalitzat.replace("", "");
		 nomNormalitzat = nomNormalitzat.replace(".l", "");
		 nomNormalitzat = nomNormalitzat.replace(".", "");
		 nomNormalitzat = nomNormalitzat.replace("ss", "s");
		 nomNormalitzat = nomNormalitzat.replace("-", "");
		 nomNormalitzat = nomNormalitzat.replace("d'", "de ");
		 nomNormalitzat = nomNormalitzat.replace("'", " ");
		 return nomNormalitzat;
	 }
	 
}
